@@ -11,18 +11,42 @@ from zarr.errors import (
     ContainsArrayError,
     ContainsGroupError,
 )
-from zarr.storage import (contains_array, contains_group, default_compressor,
-                          init_array, normalize_storage_path,
-                          normalize_store_arg)
+from zarr.storage import (
+    contains_array,
+    contains_group,
+    default_compressor,
+    init_array,
+    normalize_storage_path,
+    normalize_store_arg,
+)
 from zarr.util import normalize_dimension_separator
 
 
-def create(shape, chunks=True, dtype=None, compressor='default',
-           fill_value: Optional[int] = 0, order='C', store=None, synchronizer=None,
-           overwrite=False, path=None, chunk_store=None, filters=None,
-           cache_metadata=True, cache_attrs=True, read_only=False,
-           object_codec=None, dimension_separator=None, write_empty_chunks=True,
-           *, zarr_version=None, meta_array=None, **kwargs):
+def create(
+    shape,
+    chunks=True,
+    dtype=None,
+    compressor="default",
+    fill_value: Optional[int] = 0,
+    order="C",
+    store=None,
+    synchronizer=None,
+    overwrite=False,
+    path=None,
+    chunk_store=None,
+    filters=None,
+    cache_metadata=True,
+    cache_attrs=True,
+    read_only=False,
+    object_codec=None,
+    dimension_separator=None,
+    write_empty_chunks=True,
+    *,
+    zarr_version=None,
+    meta_array=None,
+    storage_transformers=(),
+    **kwargs,
+):
     """Create an array.
 
     Parameters
@@ -85,6 +109,14 @@ def create(shape, chunks=True, dtype=None, compressor='default',
 
         .. versionadded:: 2.11
 
+    storage_transformers : sequence of StorageTransformers, optional
+        Setting storage transformers, changes the storage structure and behaviour
+        of data coming from the underlying store. The transformers are applied in the
+        order of the given sequence. Supplying an empty sequence is the same as omitting
+        the argument or setting it to None. May only be set when using zarr_version 3.
+
+        .. versionadded:: 2.13
+
     zarr_version : {None, 2, 3}, optional
         The zarr protocol version of the created array. If None, it will be
         inferred from ``store`` or ``chunk_store`` if they are provided,
@@ -142,11 +174,11 @@ def create(shape, chunks=True, dtype=None, compressor='default',
 
     """
     if zarr_version is None and store is None:
-        zarr_version = getattr(chunk_store, '_store_version', DEFAULT_ZARR_VERSION)
+        zarr_version = getattr(chunk_store, "_store_version", DEFAULT_ZARR_VERSION)
 
     # handle polymorphic store arg
     store = normalize_store_arg(store, zarr_version=zarr_version, mode="w")
-    zarr_version = getattr(store, '_store_version', DEFAULT_ZARR_VERSION)
+    zarr_version = getattr(store, "_store_version", DEFAULT_ZARR_VERSION)
 
     # API compatibility with h5py
     compressor, fill_value = _kwargs_compat(compressor, fill_value, kwargs)
@@ -160,22 +192,43 @@ def create(shape, chunks=True, dtype=None, compressor='default',
             raise ValueError(
                 f"Specified dimension_separator: {dimension_separator}"
                 f"conflicts with store's separator: "
-                f"{store_separator}")
+                f"{store_separator}"
+            )
     dimension_separator = normalize_dimension_separator(dimension_separator)
 
     if zarr_version > 2 and path is None:
-        path = '/'
+        path = "/"
 
     # initialize array metadata
-    init_array(store, shape=shape, chunks=chunks, dtype=dtype, compressor=compressor,
-               fill_value=fill_value, order=order, overwrite=overwrite, path=path,
-               chunk_store=chunk_store, filters=filters, object_codec=object_codec,
-               dimension_separator=dimension_separator)
+    init_array(
+        store,
+        shape=shape,
+        chunks=chunks,
+        dtype=dtype,
+        compressor=compressor,
+        fill_value=fill_value,
+        order=order,
+        overwrite=overwrite,
+        path=path,
+        chunk_store=chunk_store,
+        filters=filters,
+        object_codec=object_codec,
+        dimension_separator=dimension_separator,
+        storage_transformers=storage_transformers,
+    )
 
     # instantiate array
-    z = Array(store, path=path, chunk_store=chunk_store, synchronizer=synchronizer,
-              cache_metadata=cache_metadata, cache_attrs=cache_attrs, read_only=read_only,
-              write_empty_chunks=write_empty_chunks, meta_array=meta_array)
+    z = Array(
+        store,
+        path=path,
+        chunk_store=chunk_store,
+        synchronizer=synchronizer,
+        cache_metadata=cache_metadata,
+        cache_attrs=cache_attrs,
+        read_only=read_only,
+        write_empty_chunks=write_empty_chunks,
+        meta_array=meta_array,
+    )
 
     return z
 
@@ -185,7 +238,7 @@ def _kwargs_compat(compressor, fill_value, kwargs):
     # to be compatible with h5py, as well as backwards-compatible with Zarr
     # 1.x, accept 'compression' and 'compression_opts' keyword arguments
 
-    if compressor != 'default':
+    if compressor != "default":
         # 'compressor' overrides 'compression'
         if "compression" in kwargs:
             warn(
@@ -200,14 +253,14 @@ def _kwargs_compat(compressor, fill_value, kwargs):
             )
             del kwargs["compression_opts"]
 
-    elif 'compression' in kwargs:
-        compression = kwargs.pop('compression')
-        compression_opts = kwargs.pop('compression_opts', None)
+    elif "compression" in kwargs:
+        compression = kwargs.pop("compression")
+        compression_opts = kwargs.pop("compression_opts", None)
 
-        if compression is None or compression == 'none':
+        if compression is None or compression == "none":
             compressor = None
 
-        elif compression == 'default':
+        elif compression == "default":
             compressor = default_compressor
 
         elif isinstance(compression, str):
@@ -225,21 +278,21 @@ def _kwargs_compat(compressor, fill_value, kwargs):
                 compressor = codec_cls(compression_opts)
 
         # be lenient here if user gives compressor as 'compression'
-        elif hasattr(compression, 'get_config'):
+        elif hasattr(compression, "get_config"):
             compressor = compression
 
         else:
-            raise ValueError('bad value for compression: %r' % compression)
+            raise ValueError("bad value for compression: %r" % compression)
 
     # handle 'fillvalue'
-    if 'fillvalue' in kwargs:
+    if "fillvalue" in kwargs:
         # to be compatible with h5py, accept 'fillvalue' instead of
         # 'fill_value'
-        fill_value = kwargs.pop('fillvalue')
+        fill_value = kwargs.pop("fillvalue")
 
     # ignore other keyword arguments
     for k in kwargs:
-        warn('ignoring keyword argument %r' % k)
+        warn("ignoring keyword argument %r" % k)
 
     return compressor, fill_value
 
@@ -326,16 +379,13 @@ def _get_shape_chunks(a):
     shape = None
     chunks = None
 
-    if hasattr(a, 'shape') and \
-            isinstance(a.shape, tuple):
+    if hasattr(a, "shape") and isinstance(a.shape, tuple):
         shape = a.shape
 
-        if hasattr(a, 'chunks') and \
-                isinstance(a.chunks, tuple) and \
-                (len(a.chunks) == len(a.shape)):
+        if hasattr(a, "chunks") and isinstance(a.chunks, tuple) and (len(a.chunks) == len(a.shape)):
             chunks = a.chunks
 
-        elif hasattr(a, 'chunklen'):
+        elif hasattr(a, "chunklen"):
             # bcolz carray
             chunks = (a.chunklen,) + a.shape[1:]
 
@@ -360,27 +410,27 @@ def array(data, **kwargs):
     """
 
     # ensure data is array-like
-    if not hasattr(data, 'shape') or not hasattr(data, 'dtype'):
+    if not hasattr(data, "shape") or not hasattr(data, "dtype"):
         data = np.asanyarray(data)
 
     # setup dtype
-    kw_dtype = kwargs.get('dtype')
+    kw_dtype = kwargs.get("dtype")
     if kw_dtype is None:
-        kwargs['dtype'] = data.dtype
+        kwargs["dtype"] = data.dtype
     else:
-        kwargs['dtype'] = kw_dtype
+        kwargs["dtype"] = kw_dtype
 
     # setup shape and chunks
     data_shape, data_chunks = _get_shape_chunks(data)
-    kwargs['shape'] = data_shape
-    kw_chunks = kwargs.get('chunks')
+    kwargs["shape"] = data_shape
+    kw_chunks = kwargs.get("chunks")
     if kw_chunks is None:
-        kwargs['chunks'] = data_chunks
+        kwargs["chunks"] = data_chunks
     else:
-        kwargs['chunks'] = kw_chunks
+        kwargs["chunks"] = kw_chunks
 
     # pop read-only to apply after storing the data
-    read_only = kwargs.pop('read_only', False)
+    read_only = kwargs.pop("read_only", False)
 
     # instantiate array
     z = create(**kwargs)
@@ -416,7 +466,8 @@ def open_array(
     *,
     zarr_version=None,
     dimension_separator=None,
-    **kwargs
+    meta_array=None,
+    **kwargs,
 ):
     """Open an array using file-mode-like semantics.
 
@@ -490,6 +541,11 @@ def open_array(
         ('/') format. If None, the appropriate value will be read from `store`
         when present. Otherwise, defaults to '.' when ``zarr_version == 2``
         and `/` otherwise.
+    meta_array : array-like, optional
+        An array instance to use for determining arrays to create and return
+        to users. Use `numpy.empty(())` by default.
+
+        .. versionadded:: 2.15
 
     Returns
     -------
@@ -525,27 +581,27 @@ def open_array(
     # a : read/write if exists, create otherwise (default)
 
     if zarr_version is None and store is None:
-        zarr_version = getattr(chunk_store, '_store_version', DEFAULT_ZARR_VERSION)
+        zarr_version = getattr(chunk_store, "_store_version", DEFAULT_ZARR_VERSION)
 
     # handle polymorphic store arg
-    store = normalize_store_arg(store, storage_options=storage_options,
-                                mode=mode, zarr_version=zarr_version)
-    zarr_version = getattr(store, '_store_version', DEFAULT_ZARR_VERSION)
+    store = normalize_store_arg(
+        store, storage_options=storage_options, mode=mode, zarr_version=zarr_version
+    )
+    zarr_version = getattr(store, "_store_version", DEFAULT_ZARR_VERSION)
     if chunk_store is not None:
-        chunk_store = normalize_store_arg(chunk_store,
-                                          storage_options=storage_options,
-                                          mode=mode,
-                                          zarr_version=zarr_version)
+        chunk_store = normalize_store_arg(
+            chunk_store, storage_options=storage_options, mode=mode, zarr_version=zarr_version
+        )
 
     # respect the dimension separator specified in a store, if present
     if dimension_separator is None:
-        if hasattr(store, '_dimension_separator'):
+        if hasattr(store, "_dimension_separator"):
             dimension_separator = store._dimension_separator
         else:
-            dimension_separator = '.' if zarr_version == 2 else '/'
+            dimension_separator = "." if zarr_version == 2 else "/"
 
     if zarr_version == 3 and path is None:
-        path = 'array'  # TODO: raise ValueError instead?
+        path = "array"  # TODO: raise ValueError instead?
 
     path = normalize_storage_path(path)
 
@@ -558,48 +614,84 @@ def open_array(
 
     # ensure store is initialized
 
-    if mode in ['r', 'r+']:
+    if mode in ["r", "r+"]:
         if not contains_array(store, path=path):
             if contains_group(store, path=path):
                 raise ContainsGroupError(path)
             raise ArrayNotFoundError(path)
 
-    elif mode == 'w':
-        init_array(store, shape=shape, chunks=chunks, dtype=dtype,
-                   compressor=compressor, fill_value=fill_value,
-                   order=order, filters=filters, overwrite=True, path=path,
-                   object_codec=object_codec, chunk_store=chunk_store,
-                   dimension_separator=dimension_separator)
+    elif mode == "w":
+        init_array(
+            store,
+            shape=shape,
+            chunks=chunks,
+            dtype=dtype,
+            compressor=compressor,
+            fill_value=fill_value,
+            order=order,
+            filters=filters,
+            overwrite=True,
+            path=path,
+            object_codec=object_codec,
+            chunk_store=chunk_store,
+            dimension_separator=dimension_separator,
+        )
 
-    elif mode == 'a':
+    elif mode == "a":
         if not contains_array(store, path=path):
             if contains_group(store, path=path):
                 raise ContainsGroupError(path)
-            init_array(store, shape=shape, chunks=chunks, dtype=dtype,
-                       compressor=compressor, fill_value=fill_value,
-                       order=order, filters=filters, path=path,
-                       object_codec=object_codec, chunk_store=chunk_store,
-                       dimension_separator=dimension_separator)
+            init_array(
+                store,
+                shape=shape,
+                chunks=chunks,
+                dtype=dtype,
+                compressor=compressor,
+                fill_value=fill_value,
+                order=order,
+                filters=filters,
+                path=path,
+                object_codec=object_codec,
+                chunk_store=chunk_store,
+                dimension_separator=dimension_separator,
+            )
 
-    elif mode in ['w-', 'x']:
+    elif mode in ["w-", "x"]:
         if contains_group(store, path=path):
             raise ContainsGroupError(path)
         elif contains_array(store, path=path):
             raise ContainsArrayError(path)
         else:
-            init_array(store, shape=shape, chunks=chunks, dtype=dtype,
-                       compressor=compressor, fill_value=fill_value,
-                       order=order, filters=filters, path=path,
-                       object_codec=object_codec, chunk_store=chunk_store,
-                       dimension_separator=dimension_separator)
+            init_array(
+                store,
+                shape=shape,
+                chunks=chunks,
+                dtype=dtype,
+                compressor=compressor,
+                fill_value=fill_value,
+                order=order,
+                filters=filters,
+                path=path,
+                object_codec=object_codec,
+                chunk_store=chunk_store,
+                dimension_separator=dimension_separator,
+            )
 
     # determine read only status
-    read_only = mode == 'r'
+    read_only = mode == "r"
 
     # instantiate array
-    z = Array(store, read_only=read_only, synchronizer=synchronizer,
-              cache_metadata=cache_metadata, cache_attrs=cache_attrs, path=path,
-              chunk_store=chunk_store, write_empty_chunks=write_empty_chunks)
+    z = Array(
+        store,
+        read_only=read_only,
+        synchronizer=synchronizer,
+        cache_metadata=cache_metadata,
+        cache_attrs=cache_attrs,
+        path=path,
+        chunk_store=chunk_store,
+        write_empty_chunks=write_empty_chunks,
+        meta_array=meta_array,
+    )
 
     return z
 
@@ -608,21 +700,21 @@ def _like_args(a, kwargs):
 
     shape, chunks = _get_shape_chunks(a)
     if shape is not None:
-        kwargs.setdefault('shape', shape)
+        kwargs.setdefault("shape", shape)
     if chunks is not None:
-        kwargs.setdefault('chunks', chunks)
+        kwargs.setdefault("chunks", chunks)
 
-    if hasattr(a, 'dtype'):
-        kwargs.setdefault('dtype', a.dtype)
+    if hasattr(a, "dtype"):
+        kwargs.setdefault("dtype", a.dtype)
 
     if isinstance(a, Array):
-        kwargs.setdefault('compressor', a.compressor)
-        kwargs.setdefault('order', a.order)
-        kwargs.setdefault('filters', a.filters)
-        kwargs.setdefault('zarr_version', a._version)
+        kwargs.setdefault("compressor", a.compressor)
+        kwargs.setdefault("order", a.order)
+        kwargs.setdefault("filters", a.filters)
+        kwargs.setdefault("zarr_version", a._version)
     else:
-        kwargs.setdefault('compressor', 'default')
-        kwargs.setdefault('order', 'C')
+        kwargs.setdefault("compressor", "default")
+        kwargs.setdefault("order", "C")
 
 
 def empty_like(a, **kwargs):
@@ -647,7 +739,7 @@ def full_like(a, **kwargs):
     """Create a filled array like `a`."""
     _like_args(a, kwargs)
     if isinstance(a, Array):
-        kwargs.setdefault('fill_value', a.fill_value)
+        kwargs.setdefault("fill_value", a.fill_value)
     return full(**kwargs)
 
 
@@ -655,5 +747,5 @@ def open_like(a, path, **kwargs):
     """Open a persistent array like `a`."""
     _like_args(a, kwargs)
     if isinstance(a, Array):
-        kwargs.setdefault('fill_value', a.fill_value)
+        kwargs.setdefault("fill_value", a.fill_value)
     return open_array(path, **kwargs)
