@@ -3,12 +3,27 @@ from itertools import islice
 
 import numpy as np
 
-from zarr._storage.store import (_get_metadata_suffix, data_root, meta_root,
-                                 DEFAULT_ZARR_VERSION, assert_zarr_v3_api_available)
+from zarr._storage.store import (
+    _get_metadata_suffix,
+    data_root,
+    meta_root,
+    DEFAULT_ZARR_VERSION,
+    assert_zarr_v3_api_available,
+)
 from zarr.attrs import Attributes
 from zarr.core import Array
-from zarr.creation import (array, create, empty, empty_like, full, full_like,
-                           ones, ones_like, zeros, zeros_like)
+from zarr.creation import (
+    array,
+    create,
+    empty,
+    empty_like,
+    full,
+    full_like,
+    ones,
+    ones_like,
+    zeros,
+    zeros_like,
+)
 from zarr.errors import (
     ContainsArrayError,
     ContainsGroupError,
@@ -120,12 +135,21 @@ class Group(MutableMapping):
 
     """
 
-    def __init__(self, store, path=None, read_only=False, chunk_store=None,
-                 cache_attrs=True, synchronizer=None, zarr_version=None, *,
-                 meta_array=None):
+    def __init__(
+        self,
+        store,
+        path=None,
+        read_only=False,
+        chunk_store=None,
+        cache_attrs=True,
+        synchronizer=None,
+        zarr_version=None,
+        *,
+        meta_array=None
+    ):
         store: BaseStore = _normalize_store_arg(store, zarr_version=zarr_version)
         if zarr_version is None:
-            zarr_version = getattr(store, '_store_version', DEFAULT_ZARR_VERSION)
+            zarr_version = getattr(store, "_store_version", DEFAULT_ZARR_VERSION)
 
         if zarr_version != 2:
             assert_zarr_v3_api_available()
@@ -136,9 +160,9 @@ class Group(MutableMapping):
         self._chunk_store = chunk_store
         self._path = normalize_storage_path(path)
         if self._path:
-            self._key_prefix = self._path + '/'
+            self._key_prefix = self._path + "/"
         else:
-            self._key_prefix = ''
+            self._key_prefix = ""
         self._read_only = read_only
         self._synchronizer = synchronizer
         if meta_array is not None:
@@ -182,8 +206,9 @@ class Group(MutableMapping):
             # Note: mkey doesn't actually exist for implicit groups, but the
             # object can still be created.
             akey = mkey
-        self._attrs = Attributes(store, key=akey, read_only=read_only,
-                                 cache=cache_attrs, synchronizer=synchronizer)
+        self._attrs = Attributes(
+            store, key=akey, read_only=read_only, cache=cache_attrs, synchronizer=synchronizer
+        )
 
         # setup info
         self._info = InfoReporter(self)
@@ -204,15 +229,15 @@ class Group(MutableMapping):
         if self._path:
             # follow h5py convention: add leading slash
             name = self._path
-            if name[0] != '/':
-                name = '/' + name
+            if name[0] != "/":
+                name = "/" + name
             return name
-        return '/'
+        return "/"
 
     @property
     def basename(self):
         """Final component of name."""
-        return self.name.split('/')[-1]
+        return self.name.split("/")[-1]
 
     @property
     def read_only(self):
@@ -252,10 +277,10 @@ class Group(MutableMapping):
 
     def __eq__(self, other):
         return (
-            isinstance(other, Group) and
-            self._store == other.store and
-            self._read_only == other.read_only and
-            self._path == other.path
+            isinstance(other, Group)
+            and self._store == other.store
+            and self._read_only == other.read_only
+            and self._path == other.path
             # N.B., no need to compare attributes, should be covered by
             # store comparison
         )
@@ -279,11 +304,10 @@ class Group(MutableMapping):
         quux
 
         """
-        if getattr(self._store, '_store_version', 2) == 2:
+        if getattr(self._store, "_store_version", 2) == 2:
             for key in sorted(listdir(self._store, self._path)):
                 path = self._key_prefix + key
-                if (contains_array(self._store, path) or
-                        contains_group(self._store, path)):
+                if contains_array(self._store, path) or contains_group(self._store, path):
                     yield key
         else:
             # TODO: Should this iterate over data folders and/or metadata
@@ -296,15 +320,15 @@ class Group(MutableMapping):
             # yield any groups or arrays
             sfx = self._metadata_key_suffix
             for key in keys:
-                len_suffix = len('.group') + len(sfx)  # same for .array
-                if key.endswith(('.group' + sfx, '.array' + sfx)):
+                len_suffix = len(".group") + len(sfx)  # same for .array
+                if key.endswith((".group" + sfx, ".array" + sfx)):
                     yield key[name_start:-len_suffix]
 
             # also yield any implicit groups
             for prefix in prefixes:
-                prefix = prefix.rstrip('/')
+                prefix = prefix.rstrip("/")
                 # only implicit if there is no .group.sfx file
-                if not prefix + '.group' + sfx in self._store:
+                if prefix + ".group" + sfx not in self._store:
                     yield prefix[name_start:]
 
             # Note: omit data/root/ to avoid duplicate listings
@@ -316,12 +340,12 @@ class Group(MutableMapping):
 
     def __repr__(self):
         t = type(self)
-        r = '<{}.{}'.format(t.__module__, t.__name__)
+        r = "<{}.{}".format(t.__module__, t.__name__)
         if self.name:
-            r += ' %r' % self.name
+            r += " %r" % self.name
         if self._read_only:
-            r += ' read-only'
-        r += '>'
+            r += " read-only"
+        r += ">"
         return r
 
     def __enter__(self):
@@ -333,39 +357,38 @@ class Group(MutableMapping):
         self.store.close()
 
     def info_items(self):
-
         def typestr(o):
-            return '{}.{}'.format(type(o).__module__, type(o).__name__)
+            return "{}.{}".format(type(o).__module__, type(o).__name__)
 
         items = []
 
         # basic info
         if self.name is not None:
-            items += [('Name', self.name)]
+            items += [("Name", self.name)]
         items += [
-            ('Type', typestr(self)),
-            ('Read-only', str(self.read_only)),
+            ("Type", typestr(self)),
+            ("Read-only", str(self.read_only)),
         ]
 
         # synchronizer
         if self._synchronizer is not None:
-            items += [('Synchronizer type', typestr(self._synchronizer))]
+            items += [("Synchronizer type", typestr(self._synchronizer))]
 
         # storage info
-        items += [('Store type', typestr(self._store))]
+        items += [("Store type", typestr(self._store))]
         if self._chunk_store is not None:
-            items += [('Chunk store type', typestr(self._chunk_store))]
+            items += [("Chunk store type", typestr(self._chunk_store))]
 
         # members
-        items += [('No. members', len(self))]
+        items += [("No. members", len(self))]
         array_keys = sorted(self.array_keys())
         group_keys = sorted(self.group_keys())
-        items += [('No. arrays', len(array_keys))]
-        items += [('No. groups', len(group_keys))]
+        items += [("No. arrays", len(array_keys))]
+        items += [("No. groups", len(group_keys))]
         if array_keys:
-            items += [('Arrays', ', '.join(array_keys))]
+            items += [("Arrays", ", ".join(array_keys))]
         if group_keys:
-            items += [('Groups', ', '.join(group_keys))]
+            items += [("Groups", ", ".join(group_keys))]
 
         return items
 
@@ -385,7 +408,7 @@ class Group(MutableMapping):
         self.__init__(**state)
 
     def _item_path(self, item):
-        absolute = isinstance(item, str) and item and item[0] == '/'
+        absolute = isinstance(item, str) and item and item[0] == "/"
         path = normalize_storage_path(item)
         if not absolute and self._path:
             path = self._key_prefix + path
@@ -409,8 +432,9 @@ class Group(MutableMapping):
 
         """
         path = self._item_path(item)
-        return contains_array(self._store, path) or \
-            contains_group(self._store, path, explicit_only=False)
+        return contains_array(self._store, path) or contains_group(
+            self._store, path, explicit_only=False
+        )
 
     def __getitem__(self, item):
         """Obtain a group member.
@@ -435,23 +459,41 @@ class Group(MutableMapping):
         """
         path = self._item_path(item)
         if contains_array(self._store, path):
-            return Array(self._store, read_only=self._read_only, path=path,
-                         chunk_store=self._chunk_store,
-                         synchronizer=self._synchronizer, cache_attrs=self.attrs.cache,
-                         zarr_version=self._version, meta_array=self._meta_array)
+            return Array(
+                self._store,
+                read_only=self._read_only,
+                path=path,
+                chunk_store=self._chunk_store,
+                synchronizer=self._synchronizer,
+                cache_attrs=self.attrs.cache,
+                zarr_version=self._version,
+                meta_array=self._meta_array,
+            )
         elif contains_group(self._store, path, explicit_only=True):
-            return Group(self._store, read_only=self._read_only, path=path,
-                         chunk_store=self._chunk_store, cache_attrs=self.attrs.cache,
-                         synchronizer=self._synchronizer, zarr_version=self._version,
-                         meta_array=self._meta_array)
+            return Group(
+                self._store,
+                read_only=self._read_only,
+                path=path,
+                chunk_store=self._chunk_store,
+                cache_attrs=self.attrs.cache,
+                synchronizer=self._synchronizer,
+                zarr_version=self._version,
+                meta_array=self._meta_array,
+            )
         elif self._version == 3:
-            implicit_group = meta_root + path + '/'
+            implicit_group = meta_root + path + "/"
             # non-empty folder in the metadata path implies an implicit group
             if self._store.list_prefix(implicit_group):
-                return Group(self._store, read_only=self._read_only, path=path,
-                             chunk_store=self._chunk_store, cache_attrs=self.attrs.cache,
-                             synchronizer=self._synchronizer, zarr_version=self._version,
-                             meta_array=self._meta_array)
+                return Group(
+                    self._store,
+                    read_only=self._read_only,
+                    path=path,
+                    chunk_store=self._chunk_store,
+                    cache_attrs=self.attrs.cache,
+                    synchronizer=self._synchronizer,
+                    zarr_version=self._version,
+                    meta_array=self._meta_array,
+                )
             else:
                 raise KeyError(item)
         else:
@@ -465,8 +507,9 @@ class Group(MutableMapping):
 
     def _delitem_nosync(self, item):
         path = self._item_path(item)
-        if contains_array(self._store, path) or \
-                contains_group(self._store, path, explicit_only=False):
+        if contains_array(self._store, path) or contains_group(
+            self._store, path, explicit_only=False
+        ):
             rmdir(self._store, path)
         else:
             raise KeyError(item)
@@ -510,13 +553,13 @@ class Group(MutableMapping):
                     yield key
         else:
             dir_name = meta_root + self._path
-            group_sfx = '.group' + self._metadata_key_suffix
+            group_sfx = ".group" + self._metadata_key_suffix
             # The fact that we call sorted means this can't be a streaming generator.
             # The keys are already in memory.
             all_keys = sorted(listdir(self._store, dir_name))
             for key in all_keys:
                 if key.endswith(group_sfx):
-                    key = key[:-len(group_sfx)]
+                    key = key[: -len(group_sfx)]
                     if key in all_keys:
                         # otherwise we will double count this group
                         continue
@@ -555,7 +598,8 @@ class Group(MutableMapping):
                         chunk_store=self._chunk_store,
                         cache_attrs=self.attrs.cache,
                         synchronizer=self._synchronizer,
-                        zarr_version=self._version)
+                        zarr_version=self._version,
+                    )
 
         else:
             for key in self.group_keys():
@@ -567,7 +611,8 @@ class Group(MutableMapping):
                     chunk_store=self._chunk_store,
                     cache_attrs=self.attrs.cache,
                     synchronizer=self._synchronizer,
-                    zarr_version=self._version)
+                    zarr_version=self._version,
+                )
 
     def array_keys(self, recurse=False):
         """Return an iterator over member names for arrays only.
@@ -591,9 +636,7 @@ class Group(MutableMapping):
         ['baz', 'quux']
 
         """
-        return self._array_iter(keys_only=True,
-                                method='array_keys',
-                                recurse=recurse)
+        return self._array_iter(keys_only=True, method="array_keys", recurse=recurse)
 
     def arrays(self, recurse=False):
         """Return an iterator over (name, value) pairs for arrays only.
@@ -619,9 +662,7 @@ class Group(MutableMapping):
         quux <class 'zarr.core.Array'>
 
         """
-        return self._array_iter(keys_only=False,
-                                method='arrays',
-                                recurse=recurse)
+        return self._array_iter(keys_only=False, method="arrays", recurse=recurse)
 
     def _array_iter(self, keys_only, method, recurse):
         if self._version == 2:
@@ -635,12 +676,12 @@ class Group(MutableMapping):
                     yield from getattr(group, method)(recurse=recurse)
         else:
             dir_name = meta_root + self._path
-            array_sfx = '.array' + self._metadata_key_suffix
-            group_sfx = '.group' + self._metadata_key_suffix
+            array_sfx = ".array" + self._metadata_key_suffix
+            group_sfx = ".group" + self._metadata_key_suffix
 
             for key in sorted(listdir(self._store, dir_name)):
                 if key.endswith(array_sfx):
-                    key = key[:-len(array_sfx)]
+                    key = key[: -len(array_sfx)]
                     _key = key.rstrip("/")
                     yield _key if keys_only else (_key, self[key])
 
@@ -794,8 +835,7 @@ class Group(MutableMapping):
         return self.visitvalues(lambda o: func(o.name[base_len:].lstrip("/")))
 
     def visitkeys(self, func):
-        """An alias for :py:meth:`~Group.visit`.
-        """
+        """An alias for :py:meth:`~Group.visit`."""
 
         return self.visit(func)
 
@@ -924,12 +964,17 @@ class Group(MutableMapping):
         path = self._item_path(name)
 
         # create terminal group
-        init_group(self._store, path=path, chunk_store=self._chunk_store,
-                   overwrite=overwrite)
+        init_group(self._store, path=path, chunk_store=self._chunk_store, overwrite=overwrite)
 
-        return Group(self._store, path=path, read_only=self._read_only,
-                     chunk_store=self._chunk_store, cache_attrs=self.attrs.cache,
-                     synchronizer=self._synchronizer, zarr_version=self._version)
+        return Group(
+            self._store,
+            path=path,
+            read_only=self._read_only,
+            chunk_store=self._chunk_store,
+            cache_attrs=self.attrs.cache,
+            synchronizer=self._synchronizer,
+            zarr_version=self._version,
+        )
 
     def create_groups(self, *names, **kwargs):
         """Convenience method to create multiple groups in a single call."""
@@ -960,20 +1005,26 @@ class Group(MutableMapping):
 
         """
 
-        return self._write_op(self._require_group_nosync, name,
-                              overwrite=overwrite)
+        return self._write_op(self._require_group_nosync, name, overwrite=overwrite)
 
     def _require_group_nosync(self, name, overwrite=False):
         path = self._item_path(name)
 
         # create terminal group if necessary
         if not contains_group(self._store, path):
-            init_group(store=self._store, path=path, chunk_store=self._chunk_store,
-                       overwrite=overwrite)
+            init_group(
+                store=self._store, path=path, chunk_store=self._chunk_store, overwrite=overwrite
+            )
 
-        return Group(self._store, path=path, read_only=self._read_only,
-                     chunk_store=self._chunk_store, cache_attrs=self.attrs.cache,
-                     synchronizer=self._synchronizer, zarr_version=self._version)
+        return Group(
+            self._store,
+            path=path,
+            read_only=self._read_only,
+            chunk_store=self._chunk_store,
+            cache_attrs=self.attrs.cache,
+            synchronizer=self._synchronizer,
+            zarr_version=self._version,
+        )
 
     def require_groups(self, *names):
         """Convenience method to require multiple groups in a single call."""
@@ -1048,17 +1099,15 @@ class Group(MutableMapping):
         path = self._item_path(name)
 
         # determine synchronizer
-        kwargs.setdefault('synchronizer', self._synchronizer)
-        kwargs.setdefault('cache_attrs', self.attrs.cache)
+        kwargs.setdefault("synchronizer", self._synchronizer)
+        kwargs.setdefault("cache_attrs", self.attrs.cache)
 
         # create array
         if data is None:
-            a = create(store=self._store, path=path, chunk_store=self._chunk_store,
-                       **kwargs)
+            a = create(store=self._store, path=path, chunk_store=self._chunk_store, **kwargs)
 
         else:
-            a = array(data, store=self._store, path=path, chunk_store=self._chunk_store,
-                      **kwargs)
+            a = array(data, store=self._store, path=path, chunk_store=self._chunk_store, **kwargs)
 
         return a
 
@@ -1084,11 +1133,11 @@ class Group(MutableMapping):
 
         """
 
-        return self._write_op(self._require_dataset_nosync, name, shape=shape,
-                              dtype=dtype, exact=exact, **kwargs)
+        return self._write_op(
+            self._require_dataset_nosync, name, shape=shape, dtype=dtype, exact=exact, **kwargs
+        )
 
-    def _require_dataset_nosync(self, name, shape, dtype=None, exact=False,
-                                **kwargs):
+    def _require_dataset_nosync(self, name, shape, dtype=None, exact=False, **kwargs):
 
         path = self._item_path(name)
 
@@ -1096,31 +1145,37 @@ class Group(MutableMapping):
 
             # array already exists at path, validate that it is the right shape and type
 
-            synchronizer = kwargs.get('synchronizer', self._synchronizer)
-            cache_metadata = kwargs.get('cache_metadata', True)
-            cache_attrs = kwargs.get('cache_attrs', self.attrs.cache)
-            a = Array(self._store, path=path, read_only=self._read_only,
-                      chunk_store=self._chunk_store, synchronizer=synchronizer,
-                      cache_metadata=cache_metadata, cache_attrs=cache_attrs,
-                      meta_array=self._meta_array)
+            synchronizer = kwargs.get("synchronizer", self._synchronizer)
+            cache_metadata = kwargs.get("cache_metadata", True)
+            cache_attrs = kwargs.get("cache_attrs", self.attrs.cache)
+            a = Array(
+                self._store,
+                path=path,
+                read_only=self._read_only,
+                chunk_store=self._chunk_store,
+                synchronizer=synchronizer,
+                cache_metadata=cache_metadata,
+                cache_attrs=cache_attrs,
+                meta_array=self._meta_array,
+            )
             shape = normalize_shape(shape)
             if shape != a.shape:
-                raise TypeError('shape do not match existing array; expected {}, got {}'
-                                .format(a.shape, shape))
+                raise TypeError(
+                    "shape do not match existing array; expected {}, got {}".format(a.shape, shape)
+                )
             dtype = np.dtype(dtype)
             if exact:
                 if dtype != a.dtype:
-                    raise TypeError('dtypes do not match exactly; expected {}, got {}'
-                                    .format(a.dtype, dtype))
+                    raise TypeError(
+                        "dtypes do not match exactly; expected {}, got {}".format(a.dtype, dtype)
+                    )
             else:
                 if not np.can_cast(dtype, a.dtype):
-                    raise TypeError('dtypes ({}, {}) cannot be safely cast'
-                                    .format(dtype, a.dtype))
+                    raise TypeError("dtypes ({}, {}) cannot be safely cast".format(dtype, a.dtype))
             return a
 
         else:
-            return self._create_dataset_nosync(name, shape=shape, dtype=dtype,
-                                               **kwargs)
+            return self._create_dataset_nosync(name, shape=shape, dtype=dtype, **kwargs)
 
     def create(self, name, **kwargs):
         """Create an array. Keyword arguments as per
@@ -1129,10 +1184,9 @@ class Group(MutableMapping):
 
     def _create_nosync(self, name, **kwargs):
         path = self._item_path(name)
-        kwargs.setdefault('synchronizer', self._synchronizer)
-        kwargs.setdefault('cache_attrs', self.attrs.cache)
-        return create(store=self._store, path=path, chunk_store=self._chunk_store,
-                      **kwargs)
+        kwargs.setdefault("synchronizer", self._synchronizer)
+        kwargs.setdefault("cache_attrs", self.attrs.cache)
+        return create(store=self._store, path=path, chunk_store=self._chunk_store, **kwargs)
 
     def empty(self, name, **kwargs):
         """Create an array. Keyword arguments as per
@@ -1141,10 +1195,9 @@ class Group(MutableMapping):
 
     def _empty_nosync(self, name, **kwargs):
         path = self._item_path(name)
-        kwargs.setdefault('synchronizer', self._synchronizer)
-        kwargs.setdefault('cache_attrs', self.attrs.cache)
-        return empty(store=self._store, path=path, chunk_store=self._chunk_store,
-                     **kwargs)
+        kwargs.setdefault("synchronizer", self._synchronizer)
+        kwargs.setdefault("cache_attrs", self.attrs.cache)
+        return empty(store=self._store, path=path, chunk_store=self._chunk_store, **kwargs)
 
     def zeros(self, name, **kwargs):
         """Create an array. Keyword arguments as per
@@ -1153,10 +1206,9 @@ class Group(MutableMapping):
 
     def _zeros_nosync(self, name, **kwargs):
         path = self._item_path(name)
-        kwargs.setdefault('synchronizer', self._synchronizer)
-        kwargs.setdefault('cache_attrs', self.attrs.cache)
-        return zeros(store=self._store, path=path, chunk_store=self._chunk_store,
-                     **kwargs)
+        kwargs.setdefault("synchronizer", self._synchronizer)
+        kwargs.setdefault("cache_attrs", self.attrs.cache)
+        return zeros(store=self._store, path=path, chunk_store=self._chunk_store, **kwargs)
 
     def ones(self, name, **kwargs):
         """Create an array. Keyword arguments as per
@@ -1165,8 +1217,8 @@ class Group(MutableMapping):
 
     def _ones_nosync(self, name, **kwargs):
         path = self._item_path(name)
-        kwargs.setdefault('synchronizer', self._synchronizer)
-        kwargs.setdefault('cache_attrs', self.attrs.cache)
+        kwargs.setdefault("synchronizer", self._synchronizer)
+        kwargs.setdefault("cache_attrs", self.attrs.cache)
         return ones(store=self._store, path=path, chunk_store=self._chunk_store, **kwargs)
 
     def full(self, name, fill_value, **kwargs):
@@ -1176,10 +1228,15 @@ class Group(MutableMapping):
 
     def _full_nosync(self, name, fill_value, **kwargs):
         path = self._item_path(name)
-        kwargs.setdefault('synchronizer', self._synchronizer)
-        kwargs.setdefault('cache_attrs', self.attrs.cache)
-        return full(store=self._store, path=path, chunk_store=self._chunk_store,
-                    fill_value=fill_value, **kwargs)
+        kwargs.setdefault("synchronizer", self._synchronizer)
+        kwargs.setdefault("cache_attrs", self.attrs.cache)
+        return full(
+            store=self._store,
+            path=path,
+            chunk_store=self._chunk_store,
+            fill_value=fill_value,
+            **kwargs
+        )
 
     def array(self, name, data, **kwargs):
         """Create an array. Keyword arguments as per
@@ -1188,10 +1245,9 @@ class Group(MutableMapping):
 
     def _array_nosync(self, name, data, **kwargs):
         path = self._item_path(name)
-        kwargs.setdefault('synchronizer', self._synchronizer)
-        kwargs.setdefault('cache_attrs', self.attrs.cache)
-        return array(data, store=self._store, path=path, chunk_store=self._chunk_store,
-                     **kwargs)
+        kwargs.setdefault("synchronizer", self._synchronizer)
+        kwargs.setdefault("cache_attrs", self.attrs.cache)
+        return array(data, store=self._store, path=path, chunk_store=self._chunk_store, **kwargs)
 
     def empty_like(self, name, data, **kwargs):
         """Create an array. Keyword arguments as per
@@ -1200,10 +1256,11 @@ class Group(MutableMapping):
 
     def _empty_like_nosync(self, name, data, **kwargs):
         path = self._item_path(name)
-        kwargs.setdefault('synchronizer', self._synchronizer)
-        kwargs.setdefault('cache_attrs', self.attrs.cache)
-        return empty_like(data, store=self._store, path=path,
-                          chunk_store=self._chunk_store, **kwargs)
+        kwargs.setdefault("synchronizer", self._synchronizer)
+        kwargs.setdefault("cache_attrs", self.attrs.cache)
+        return empty_like(
+            data, store=self._store, path=path, chunk_store=self._chunk_store, **kwargs
+        )
 
     def zeros_like(self, name, data, **kwargs):
         """Create an array. Keyword arguments as per
@@ -1212,10 +1269,11 @@ class Group(MutableMapping):
 
     def _zeros_like_nosync(self, name, data, **kwargs):
         path = self._item_path(name)
-        kwargs.setdefault('synchronizer', self._synchronizer)
-        kwargs.setdefault('cache_attrs', self.attrs.cache)
-        return zeros_like(data, store=self._store, path=path,
-                          chunk_store=self._chunk_store, **kwargs)
+        kwargs.setdefault("synchronizer", self._synchronizer)
+        kwargs.setdefault("cache_attrs", self.attrs.cache)
+        return zeros_like(
+            data, store=self._store, path=path, chunk_store=self._chunk_store, **kwargs
+        )
 
     def ones_like(self, name, data, **kwargs):
         """Create an array. Keyword arguments as per
@@ -1224,10 +1282,11 @@ class Group(MutableMapping):
 
     def _ones_like_nosync(self, name, data, **kwargs):
         path = self._item_path(name)
-        kwargs.setdefault('synchronizer', self._synchronizer)
-        kwargs.setdefault('cache_attrs', self.attrs.cache)
-        return ones_like(data, store=self._store, path=path,
-                         chunk_store=self._chunk_store, **kwargs)
+        kwargs.setdefault("synchronizer", self._synchronizer)
+        kwargs.setdefault("cache_attrs", self.attrs.cache)
+        return ones_like(
+            data, store=self._store, path=path, chunk_store=self._chunk_store, **kwargs
+        )
 
     def full_like(self, name, data, **kwargs):
         """Create an array. Keyword arguments as per
@@ -1236,10 +1295,11 @@ class Group(MutableMapping):
 
     def _full_like_nosync(self, name, data, **kwargs):
         path = self._item_path(name)
-        kwargs.setdefault('synchronizer', self._synchronizer)
-        kwargs.setdefault('cache_attrs', self.attrs.cache)
-        return full_like(data, store=self._store, path=path,
-                         chunk_store=self._chunk_store, **kwargs)
+        kwargs.setdefault("synchronizer", self._synchronizer)
+        kwargs.setdefault("cache_attrs", self.attrs.cache)
+        return full_like(
+            data, store=self._store, path=path, chunk_store=self._chunk_store, **kwargs
+        )
 
     def _move_nosync(self, path, new_path):
         rename(self._store, path, new_path)
@@ -1261,11 +1321,14 @@ class Group(MutableMapping):
         dest = self._item_path(dest)
 
         # Check that source exists.
-        if not (contains_array(self._store, source) or
-                contains_group(self._store, source, explicit_only=False)):
+        if not (
+            contains_array(self._store, source)
+            or contains_group(self._store, source, explicit_only=False)
+        ):
             raise ValueError('The source, "%s", does not exist.' % source)
-        if (contains_array(self._store, dest) or
-                contains_group(self._store, dest, explicit_only=False)):
+        if contains_array(self._store, dest) or contains_group(
+            self._store, dest, explicit_only=False
+        ):
             raise ValueError('The dest, "%s", already exists.' % dest)
 
         # Ensure groups needed for `dest` exist.
@@ -1275,23 +1338,30 @@ class Group(MutableMapping):
         self._write_op(self._move_nosync, source, dest)
 
 
-def _normalize_store_arg(store, *, storage_options=None, mode="r",
-                         zarr_version=None):
+def _normalize_store_arg(store, *, storage_options=None, mode="r", zarr_version=None):
     if zarr_version is None:
-        zarr_version = getattr(store, '_store_version', DEFAULT_ZARR_VERSION)
+        zarr_version = getattr(store, "_store_version", DEFAULT_ZARR_VERSION)
 
     if zarr_version != 2:
         assert_zarr_v3_api_available()
 
     if store is None:
         return MemoryStore() if zarr_version == 2 else MemoryStoreV3()
-    return normalize_store_arg(store,
-                               storage_options=storage_options, mode=mode,
-                               zarr_version=zarr_version)
+    return normalize_store_arg(
+        store, storage_options=storage_options, mode=mode, zarr_version=zarr_version
+    )
 
 
-def group(store=None, overwrite=False, chunk_store=None,
-          cache_attrs=True, synchronizer=None, path=None, *, zarr_version=None):
+def group(
+    store=None,
+    overwrite=False,
+    chunk_store=None,
+    cache_attrs=True,
+    synchronizer=None,
+    path=None,
+    *,
+    zarr_version=None
+):
     """Create a group.
 
     Parameters
@@ -1336,9 +1406,9 @@ def group(store=None, overwrite=False, chunk_store=None,
     """
 
     # handle polymorphic store arg
-    store = _normalize_store_arg(store, zarr_version=zarr_version)
+    store = _normalize_store_arg(store, zarr_version=zarr_version, mode="w")
     if zarr_version is None:
-        zarr_version = getattr(store, '_store_version', DEFAULT_ZARR_VERSION)
+        zarr_version = getattr(store, "_store_version", DEFAULT_ZARR_VERSION)
 
     if zarr_version != 2:
         assert_zarr_v3_api_available()
@@ -1352,16 +1422,31 @@ def group(store=None, overwrite=False, chunk_store=None,
         requires_init = overwrite or not contains_group(store, path)
 
     if requires_init:
-        init_group(store, overwrite=overwrite, chunk_store=chunk_store,
-                   path=path)
+        init_group(store, overwrite=overwrite, chunk_store=chunk_store, path=path)
 
-    return Group(store, read_only=False, chunk_store=chunk_store,
-                 cache_attrs=cache_attrs, synchronizer=synchronizer, path=path,
-                 zarr_version=zarr_version)
+    return Group(
+        store,
+        read_only=False,
+        chunk_store=chunk_store,
+        cache_attrs=cache_attrs,
+        synchronizer=synchronizer,
+        path=path,
+        zarr_version=zarr_version,
+    )
 
 
-def open_group(store=None, mode='a', cache_attrs=True, synchronizer=None, path=None,
-               chunk_store=None, storage_options=None, *, zarr_version=None, meta_array=None):
+def open_group(
+    store=None,
+    mode="a",
+    cache_attrs=True,
+    synchronizer=None,
+    path=None,
+    chunk_store=None,
+    storage_options=None,
+    *,
+    zarr_version=None,
+    meta_array=None
+):
     """Open a group using file-mode-like semantics.
 
     Parameters
@@ -1414,44 +1499,41 @@ def open_group(store=None, mode='a', cache_attrs=True, synchronizer=None, path=N
 
     # handle polymorphic store arg
     store = _normalize_store_arg(
-        store, storage_options=storage_options, mode=mode,
-        zarr_version=zarr_version)
+        store, storage_options=storage_options, mode=mode, zarr_version=zarr_version
+    )
     if zarr_version is None:
-        zarr_version = getattr(store, '_store_version', DEFAULT_ZARR_VERSION)
+        zarr_version = getattr(store, "_store_version", DEFAULT_ZARR_VERSION)
 
     if zarr_version != 2:
         assert_zarr_v3_api_available()
 
     if chunk_store is not None:
-        chunk_store = _normalize_store_arg(chunk_store,
-                                           storage_options=storage_options,
-                                           mode=mode,
-                                           zarr_version=zarr_version)
-        if getattr(chunk_store, '_store_version', DEFAULT_ZARR_VERSION) != zarr_version:
-            raise ValueError(  # pragma: no cover
-                "zarr_version of store and chunk_store must match"
-            )
+        chunk_store = _normalize_store_arg(
+            chunk_store, storage_options=storage_options, mode=mode, zarr_version=zarr_version
+        )
+        if getattr(chunk_store, "_store_version", DEFAULT_ZARR_VERSION) != zarr_version:
+            raise ValueError("zarr_version of store and chunk_store must match")  # pragma: no cover
 
     path = normalize_storage_path(path)
 
     # ensure store is initialized
 
-    if mode in ['r', 'r+']:
+    if mode in ["r", "r+"]:
         if not contains_group(store, path=path):
             if contains_array(store, path=path):
                 raise ContainsArrayError(path)
             raise GroupNotFoundError(path)
 
-    elif mode == 'w':
+    elif mode == "w":
         init_group(store, overwrite=True, path=path, chunk_store=chunk_store)
 
-    elif mode == 'a':
+    elif mode == "a":
         if not contains_group(store, path=path):
             if contains_array(store, path=path):
                 raise ContainsArrayError(path)
             init_group(store, path=path, chunk_store=chunk_store)
 
-    elif mode in ['w-', 'x']:
+    elif mode in ["w-", "x"]:
         if contains_array(store, path=path):
             raise ContainsArrayError(path)
         elif contains_group(store, path=path):
@@ -1460,8 +1542,15 @@ def open_group(store=None, mode='a', cache_attrs=True, synchronizer=None, path=N
             init_group(store, path=path, chunk_store=chunk_store)
 
     # determine read only status
-    read_only = mode == 'r'
+    read_only = mode == "r"
 
-    return Group(store, read_only=read_only, cache_attrs=cache_attrs,
-                 synchronizer=synchronizer, path=path, chunk_store=chunk_store,
-                 zarr_version=zarr_version, meta_array=meta_array)
+    return Group(
+        store,
+        read_only=read_only,
+        cache_attrs=cache_attrs,
+        synchronizer=synchronizer,
+        path=path,
+        chunk_store=chunk_store,
+        zarr_version=zarr_version,
+        meta_array=meta_array,
+    )
