@@ -1,6 +1,15 @@
 import numpy as np
-from numcodecs import (BZ2, AsType, Blosc, Categorize, Delta, FixedScaleOffset,
-                       PackBits, Quantize, Zlib)
+from numcodecs import (
+    BZ2,
+    AsType,
+    Blosc,
+    Categorize,
+    Delta,
+    FixedScaleOffset,
+    PackBits,
+    Quantize,
+    Zlib,
+)
 from numpy.testing import assert_array_almost_equal, assert_array_equal
 
 from zarr.creation import array
@@ -23,8 +32,8 @@ else:
 def test_array_with_delta_filter():
 
     # setup
-    astype = 'u1'
-    dtype = 'i8'
+    astype = "u1"
+    dtype = "i8"
     filters = [Delta(astype=astype, dtype=dtype)]
     data = np.arange(100, dtype=dtype)
 
@@ -50,8 +59,8 @@ def test_array_with_delta_filter():
 def test_array_with_astype_filter():
 
     # setup
-    encode_dtype = 'i1'
-    decode_dtype = 'i8'
+    encode_dtype = "i1"
+    decode_dtype = "i8"
     filters = [AsType(encode_dtype=encode_dtype, decode_dtype=decode_dtype)]
     chunks = 10
     chunk_size = 10
@@ -74,18 +83,18 @@ def test_array_with_astype_filter():
             else:
                 chunk = cdata
             actual = np.frombuffer(chunk, dtype=encode_dtype)
-            expect = data.astype(encode_dtype)[i*chunk_size:(i+1)*chunk_size]
+            expect = data.astype(encode_dtype)[i * chunk_size : (i + 1) * chunk_size]
             assert_array_equal(expect, actual)
 
 
 def test_array_with_scaleoffset_filter():
 
     # setup
-    astype = 'u1'
-    dtype = 'f8'
+    astype = "u1"
+    dtype = "f8"
     flt = FixedScaleOffset(scale=10, offset=1000, astype=astype, dtype=dtype)
     filters = [flt]
-    data = np.linspace(1000, 1001, 34, dtype='f8')
+    data = np.linspace(1000, 1001, 34, dtype="f8")
 
     for compressor in compressors:
 
@@ -102,14 +111,14 @@ def test_array_with_scaleoffset_filter():
             else:
                 chunk = cdata
             actual = np.frombuffer(chunk, dtype=astype)
-            expect = flt.encode(data[i*5:(i*5)+5])
+            expect = flt.encode(data[i * 5 : (i * 5) + 5])
             assert_array_equal(expect, actual)
 
 
 def test_array_with_quantize_filter():
 
     # setup
-    dtype = 'f8'
+    dtype = "f8"
     digits = 3
     flt = Quantize(digits=digits, dtype=dtype)
     filters = [flt]
@@ -130,7 +139,7 @@ def test_array_with_quantize_filter():
             else:
                 chunk = cdata
             actual = np.frombuffer(chunk, dtype=dtype)
-            expect = flt.encode(data[i*5:(i*5)+5])
+            expect = flt.encode(data[i * 5 : (i * 5) + 5])
             assert_array_equal(expect, actual)
 
 
@@ -155,16 +164,16 @@ def test_array_with_packbits_filter():
                 chunk = compressor.decode(cdata)
             else:
                 chunk = cdata
-            actual = np.frombuffer(chunk, dtype='u1')
-            expect = flt.encode(data[i*5:(i*5)+5])
+            actual = np.frombuffer(chunk, dtype="u1")
+            expect = flt.encode(data[i * 5 : (i * 5) + 5])
             assert_array_equal(expect, actual)
 
 
 def test_array_with_categorize_filter():
 
     # setup
-    data = np.random.choice(['foo', 'bar', 'baz'], size=100)
-    flt = Categorize(dtype=data.dtype, labels=['foo', 'bar', 'baz'])
+    data = np.random.choice(["foo", "bar", "baz"], size=100)
+    flt = Categorize(dtype=data.dtype, labels=["foo", "bar", "baz"])
     filters = [flt]
 
     for compressor in compressors:
@@ -181,8 +190,8 @@ def test_array_with_categorize_filter():
                 chunk = a.compressor.decode(cdata)
             else:
                 chunk = cdata
-            actual = np.frombuffer(chunk, dtype='u1')
-            expect = flt.encode(data[i*5:(i*5)+5])
+            actual = np.frombuffer(chunk, dtype="u1")
+            expect = flt.encode(data[i * 5 : (i * 5) + 5])
             assert_array_equal(expect, actual)
 
 
@@ -193,17 +202,13 @@ def test_compressor_as_filter():
             continue
 
         # setup filters
-        dtype = 'i8'
-        filters = [
-            Delta(dtype=dtype),
-            compressor
-        ]
+        dtype = "i8"
+        filters = [Delta(dtype=dtype), compressor]
 
         # setup data and arrays
         data = np.arange(10000, dtype=dtype)
         a1 = array(data, chunks=1000, compressor=None, filters=filters)
-        a2 = array(data, chunks=1000, compressor=compressor,
-                   filters=filters[:1])
+        a2 = array(data, chunks=1000, compressor=compressor, filters=filters[:1])
 
         # check storage
         for i in range(10):
