@@ -51,22 +51,20 @@ def test_replace_ellipsis():
     assert (slice(None), 0) == replace_ellipsis((slice(None), 0), (100, 100))
 
     # 2D slice
-    assert ((slice(None), slice(None)) ==
-            replace_ellipsis(Ellipsis, (100, 100)))
-    assert ((slice(None), slice(None)) ==
-            replace_ellipsis(slice(None), (100, 100)))
-    assert ((slice(None), slice(None)) ==
-            replace_ellipsis((slice(None), slice(None)), (100, 100)))
-    assert ((slice(None), slice(None)) ==
-            replace_ellipsis((Ellipsis, slice(None)), (100, 100)))
-    assert ((slice(None), slice(None)) ==
-            replace_ellipsis((slice(None), Ellipsis), (100, 100)))
-    assert ((slice(None), slice(None)) ==
-            replace_ellipsis((slice(None), Ellipsis, slice(None)), (100, 100)))
-    assert ((slice(None), slice(None)) ==
-            replace_ellipsis((Ellipsis, slice(None), slice(None)), (100, 100)))
-    assert ((slice(None), slice(None)) ==
-            replace_ellipsis((slice(None), slice(None), Ellipsis), (100, 100)))
+    assert (slice(None), slice(None)) == replace_ellipsis(Ellipsis, (100, 100))
+    assert (slice(None), slice(None)) == replace_ellipsis(slice(None), (100, 100))
+    assert (slice(None), slice(None)) == replace_ellipsis((slice(None), slice(None)), (100, 100))
+    assert (slice(None), slice(None)) == replace_ellipsis((Ellipsis, slice(None)), (100, 100))
+    assert (slice(None), slice(None)) == replace_ellipsis((slice(None), Ellipsis), (100, 100))
+    assert (slice(None), slice(None)) == replace_ellipsis(
+        (slice(None), Ellipsis, slice(None)), (100, 100)
+    )
+    assert (slice(None), slice(None)) == replace_ellipsis(
+        (Ellipsis, slice(None), slice(None)), (100, 100)
+    )
+    assert (slice(None), slice(None)) == replace_ellipsis(
+        (slice(None), slice(None), Ellipsis), (100, 100)
+    )
 
 
 def test_get_basic_selection_0d():
@@ -87,25 +85,25 @@ def test_get_basic_selection_0d():
     assert_array_equal(a, b)
 
     # test structured array
-    value = (b'aaa', 1,  4.2)
-    a = np.array(value, dtype=[('foo', 'S3'), ('bar', 'i4'), ('baz', 'f8')])
+    value = (b"aaa", 1, 4.2)
+    a = np.array(value, dtype=[("foo", "S3"), ("bar", "i4"), ("baz", "f8")])
     z = zarr.create(shape=a.shape, dtype=a.dtype, fill_value=None)
     z[()] = value
     assert_array_equal(a, z.get_basic_selection(Ellipsis))
     assert_array_equal(a, z[...])
     assert a[()] == z.get_basic_selection(())
     assert a[()] == z[()]
-    assert b'aaa' == z.get_basic_selection((), fields='foo')
-    assert b'aaa' == z['foo']
-    assert a[['foo', 'bar']] == z.get_basic_selection((), fields=['foo', 'bar'])
-    assert a[['foo', 'bar']] == z['foo', 'bar']
+    assert b"aaa" == z.get_basic_selection((), fields="foo")
+    assert b"aaa" == z["foo"]
+    assert a[["foo", "bar"]] == z.get_basic_selection((), fields=["foo", "bar"])
+    assert a[["foo", "bar"]] == z["foo", "bar"]
     # test out param
     b = np.zeros_like(a)
     z.get_basic_selection(Ellipsis, out=b)
     assert_array_equal(a, b)
-    c = np.zeros_like(a[['foo', 'bar']])
-    z.get_basic_selection(Ellipsis, out=c, fields=['foo', 'bar'])
-    assert_array_equal(a[['foo', 'bar']], c)
+    c = np.zeros_like(a[["foo", "bar"]])
+    z.get_basic_selection(Ellipsis, out=c, fields=["foo", "bar"])
+    assert_array_equal(a[["foo", "bar"]], c)
 
 
 basic_selections_1d = [
@@ -175,8 +173,8 @@ basic_selections_1d_bad = [
     slice(-1, 0, -1),
     # bad stuff
     2.3,
-    'foo',
-    b'xxx',
+    "foo",
+    b"xxx",
     None,
     (0, 0),
     (slice(None), slice(None)),
@@ -252,8 +250,8 @@ basic_selections_2d = [
 basic_selections_2d_bad = [
     # bad stuff
     2.3,
-    'foo',
-    b'xxx',
+    "foo",
+    b"xxx",
     None,
     (2.3, slice(None)),
     # only positive step supported
@@ -283,8 +281,6 @@ def test_get_basic_selection_2d():
     for selection in bad_selections:
         with pytest.raises(IndexError):
             z.get_basic_selection(selection)
-        with pytest.raises(IndexError):
-            z[selection]
     # check fallback on fancy indexing
     fancy_selection = ([0, 1], [0, 1])
     np.testing.assert_array_equal(z[fancy_selection], [0, 11])
@@ -302,29 +298,116 @@ def test_fancy_indexing_fallback_on_get_setitem():
             [0, 0, 0, 1],
         ],
     )
-    np.testing.assert_array_equal(
-        z[[1, 2, 3], [1, 2, 3]], 1
-    )
+    np.testing.assert_array_equal(z[[1, 2, 3], [1, 2, 3]], 1)
     # test broadcasting
-    np.testing.assert_array_equal(
-        z[1, [1, 2, 3]], [1, 0, 0]
-    )
+    np.testing.assert_array_equal(z[1, [1, 2, 3]], [1, 0, 0])
     # test 1D fancy indexing
     z2 = zarr.zeros(5)
     z2[[1, 2, 3]] = 1
-    np.testing.assert_array_equal(
-        z2, [0, 1, 1, 1, 0]
-    )
+    np.testing.assert_array_equal(z2, [0, 1, 1, 1, 0])
 
 
-def test_fancy_indexing_doesnt_mix_with_slicing():
-    z = zarr.zeros((20, 20))
-    with pytest.raises(IndexError):
-        z[[1, 2, 3], :] = 2
-    with pytest.raises(IndexError):
-        np.testing.assert_array_equal(
-            z[[1, 2, 3], :], 0
-        )
+@pytest.mark.parametrize(
+    "index,expected_result",
+    [
+        # Single iterable of integers
+        ([0, 1], [[0, 1, 2], [3, 4, 5]]),
+        # List first, then slice
+        (([0, 1], slice(None)), [[0, 1, 2], [3, 4, 5]]),
+        # List first, then slice
+        (([0, 1], slice(1, None)), [[1, 2], [4, 5]]),
+        # Slice first, then list
+        ((slice(0, 2), [0, 2]), [[0, 2], [3, 5]]),
+        # Slices only
+        ((slice(0, 2), slice(0, 2)), [[0, 1], [3, 4]]),
+        # List with repeated index
+        (([1, 0, 1], slice(1, None)), [[4, 5], [1, 2], [4, 5]]),
+        # 1D indexing
+        (([1, 0, 1]), [[3, 4, 5], [0, 1, 2], [3, 4, 5]]),
+    ],
+)
+def test_orthogonal_indexing_fallback_on_getitem_2d(index, expected_result):
+    """
+    Tests the orthogonal indexing fallback on __getitem__ for a 2D matrix.
+
+    In addition to checking expected behavior, all indexing
+    is also checked against numpy.
+    """
+    # [0, 1, 2],
+    # [3, 4, 5],
+    # [6, 7, 8]
+    a = np.arange(9).reshape(3, 3)
+    z = zarr.array(a)
+
+    np.testing.assert_array_equal(z[index], a[index], err_msg="Indexing disagrees with numpy")
+    np.testing.assert_array_equal(z[index], expected_result)
+
+
+@pytest.mark.parametrize(
+    "index,expected_result",
+    [
+        # Single iterable of integers
+        ([0, 1], [[[0, 1, 2], [3, 4, 5], [6, 7, 8]], [[9, 10, 11], [12, 13, 14], [15, 16, 17]]]),
+        # One slice, two integers
+        ((slice(0, 2), 1, 1), [4, 13]),
+        # One integer, two slices
+        ((slice(0, 2), 1, slice(0, 2)), [[3, 4], [12, 13]]),
+        # Two slices and a list
+        ((slice(0, 2), [1, 2], slice(0, 2)), [[[3, 4], [6, 7]], [[12, 13], [15, 16]]]),
+    ],
+)
+def test_orthogonal_indexing_fallback_on_getitem_3d(index, expected_result):
+    """
+    Tests the orthogonal indexing fallback on __getitem__ for a 3D matrix.
+
+    In addition to checking expected behavior, all indexing
+    is also checked against numpy.
+    """
+    #   [[[ 0,  1,  2],
+    #     [ 3,  4,  5],
+    #     [ 6,  7,  8]],
+
+    #    [[ 9, 10, 11],
+    #     [12, 13, 14],
+    #     [15, 16, 17]],
+
+    #    [[18, 19, 20],
+    #     [21, 22, 23],
+    #     [24, 25, 26]]]
+    a = np.arange(27).reshape(3, 3, 3)
+    z = zarr.array(a)
+
+    np.testing.assert_array_equal(z[index], a[index], err_msg="Indexing disagrees with numpy")
+    np.testing.assert_array_equal(z[index], expected_result)
+
+
+@pytest.mark.parametrize(
+    "index,expected_result",
+    [
+        # Single iterable of integers
+        ([0, 1], [[1, 1, 1], [1, 1, 1], [0, 0, 0]]),
+        # List and slice combined
+        (([0, 1], slice(1, 3)), [[0, 1, 1], [0, 1, 1], [0, 0, 0]]),
+        # Index repetition is ignored on setitem
+        (([0, 1, 1, 1, 1, 1, 1], slice(1, 3)), [[0, 1, 1], [0, 1, 1], [0, 0, 0]]),
+        # Slice with step
+        (([0, 2], slice(None, None, 2)), [[1, 0, 1], [0, 0, 0], [1, 0, 1]]),
+    ],
+)
+def test_orthogonal_indexing_fallback_on_setitem_2d(index, expected_result):
+    """
+    Tests the orthogonal indexing fallback on __setitem__ for a 3D matrix.
+
+    In addition to checking expected behavior, all indexing
+    is also checked against numpy.
+    """
+    # Slice + fancy index
+    a = np.zeros((3, 3))
+    z = zarr.array(a)
+    z[index] = 1
+    a[index] = 1
+    np.testing.assert_array_equal(z, expected_result)
+    np.testing.assert_array_equal(z, a, err_msg="Indexing disagrees with numpy")
 
 
 def test_fancy_indexing_doesnt_mix_with_implicit_slicing():
@@ -332,21 +415,11 @@ def test_fancy_indexing_doesnt_mix_with_implicit_slicing():
     with pytest.raises(IndexError):
         z2[[1, 2, 3], [1, 2, 3]] = 2
     with pytest.raises(IndexError):
-        np.testing.assert_array_equal(
-            z2[[1, 2, 3], [1, 2, 3]], 0
-        )
-    with pytest.raises(IndexError):
-        z2[[1, 2, 3]] = 2
-    with pytest.raises(IndexError):
-        np.testing.assert_array_equal(
-            z2[[1, 2, 3]], 0
-        )
+        np.testing.assert_array_equal(z2[[1, 2, 3], [1, 2, 3]], 0)
     with pytest.raises(IndexError):
         z2[..., [1, 2, 3]] = 2
     with pytest.raises(IndexError):
-        np.testing.assert_array_equal(
-            z2[..., [1, 2, 3]], 0
-        )
+        np.testing.assert_array_equal(z2[..., [1, 2, 3]], 0)
 
 
 def test_set_basic_selection_0d():
@@ -366,8 +439,8 @@ def test_set_basic_selection_0d():
     assert_array_equal(v, z)
 
     # test structured array
-    value = (b'aaa', 1,  4.2)
-    v = np.array(value, dtype=[('foo', 'S3'), ('bar', 'i4'), ('baz', 'f8')])
+    value = (b"aaa", 1, 4.2)
+    v = np.array(value, dtype=[("foo", "S3"), ("bar", "i4"), ("baz", "f8")])
     a = np.zeros_like(v)
     z = zarr.create(shape=a.shape, dtype=a.dtype, fill_value=None)
 
@@ -381,19 +454,19 @@ def test_set_basic_selection_0d():
     z[...] = a
     assert_array_equal(a, z)
     # with fields
-    z.set_basic_selection(Ellipsis, v['foo'], fields='foo')
-    assert v['foo'] == z['foo']
-    assert a['bar'] == z['bar']
-    assert a['baz'] == z['baz']
-    z['bar'] = v['bar']
-    assert v['foo'] == z['foo']
-    assert v['bar'] == z['bar']
-    assert a['baz'] == z['baz']
+    z.set_basic_selection(Ellipsis, v["foo"], fields="foo")
+    assert v["foo"] == z["foo"]
+    assert a["bar"] == z["bar"]
+    assert a["baz"] == z["baz"]
+    z["bar"] = v["bar"]
+    assert v["foo"] == z["foo"]
+    assert v["bar"] == z["bar"]
+    assert a["baz"] == z["baz"]
     # multiple field assignment not supported
     with pytest.raises(IndexError):
-        z.set_basic_selection(Ellipsis, v[['foo', 'bar']], fields=['foo', 'bar'])
+        z.set_basic_selection(Ellipsis, v[["foo", "bar"]], fields=["foo", "bar"])
     with pytest.raises(IndexError):
-        z[..., 'foo', 'bar'] = v[['foo', 'bar']]
+        z[..., "foo", "bar"] = v[["foo", "bar"]]
 
 
 def _test_get_orthogonal_selection(a, z, selection):
@@ -453,7 +526,6 @@ def test_get_orthogonal_selection_1d_int():
         [0, 3, 10, -23, -12, -1],
         # explicit test not sorted
         [3, 105, 23, 127],
-
     ]
     for selection in selections:
         _test_get_orthogonal_selection(a, z, selection)
@@ -514,7 +586,7 @@ def test_get_orthogonal_selection_2d():
 
         # integer arrays
         ix0 = np.random.choice(a.shape[0], size=int(a.shape[0] * p), replace=True)
-        ix1 = np.random.choice(a.shape[1], size=int(a.shape[1] * .5), replace=True)
+        ix1 = np.random.choice(a.shape[1], size=int(a.shape[1] * 0.5), replace=True)
         _test_get_orthogonal_selection_2d(a, z, ix0, ix1)
         ix0.sort()
         ix1.sort()
@@ -581,14 +653,14 @@ def test_get_orthogonal_selection_3d():
 
         # boolean arrays
         ix0 = np.random.binomial(1, p, size=a.shape[0]).astype(bool)
-        ix1 = np.random.binomial(1, .5, size=a.shape[1]).astype(bool)
-        ix2 = np.random.binomial(1, .5, size=a.shape[2]).astype(bool)
+        ix1 = np.random.binomial(1, 0.5, size=a.shape[1]).astype(bool)
+        ix2 = np.random.binomial(1, 0.5, size=a.shape[2]).astype(bool)
         _test_get_orthogonal_selection_3d(a, z, ix0, ix1, ix2)
 
         # integer arrays
         ix0 = np.random.choice(a.shape[0], size=int(a.shape[0] * p), replace=True)
-        ix1 = np.random.choice(a.shape[1], size=int(a.shape[1] * .5), replace=True)
-        ix2 = np.random.choice(a.shape[2], size=int(a.shape[2] * .5), replace=True)
+        ix1 = np.random.choice(a.shape[1], size=int(a.shape[1] * 0.5), replace=True)
+        ix2 = np.random.choice(a.shape[2], size=int(a.shape[2] * 0.5), replace=True)
         _test_get_orthogonal_selection_3d(a, z, ix0, ix1, ix2)
         ix0.sort()
         ix1.sort()
@@ -689,12 +761,12 @@ def test_set_orthogonal_selection_2d():
 
         # boolean arrays
         ix0 = np.random.binomial(1, p, size=a.shape[0]).astype(bool)
-        ix1 = np.random.binomial(1, .5, size=a.shape[1]).astype(bool)
+        ix1 = np.random.binomial(1, 0.5, size=a.shape[1]).astype(bool)
         _test_set_orthogonal_selection_2d(v, a, z, ix0, ix1)
 
         # integer arrays
         ix0 = np.random.choice(a.shape[0], size=int(a.shape[0] * p), replace=True)
-        ix1 = np.random.choice(a.shape[1], size=int(a.shape[1] * .5), replace=True)
+        ix1 = np.random.choice(a.shape[1], size=int(a.shape[1] * 0.5), replace=True)
         _test_set_orthogonal_selection_2d(v, a, z, ix0, ix1)
         ix0.sort()
         ix1.sort()
@@ -747,14 +819,14 @@ def test_set_orthogonal_selection_3d():
 
         # boolean arrays
         ix0 = np.random.binomial(1, p, size=a.shape[0]).astype(bool)
-        ix1 = np.random.binomial(1, .5, size=a.shape[1]).astype(bool)
-        ix2 = np.random.binomial(1, .5, size=a.shape[2]).astype(bool)
+        ix1 = np.random.binomial(1, 0.5, size=a.shape[1]).astype(bool)
+        ix2 = np.random.binomial(1, 0.5, size=a.shape[2]).astype(bool)
         _test_set_orthogonal_selection_3d(v, a, z, ix0, ix1, ix2)
 
         # integer arrays
         ix0 = np.random.choice(a.shape[0], size=int(a.shape[0] * p), replace=True)
-        ix1 = np.random.choice(a.shape[1], size=int(a.shape[1] * .5), replace=True)
-        ix2 = np.random.choice(a.shape[2], size=int(a.shape[2] * .5), replace=True)
+        ix1 = np.random.choice(a.shape[1], size=int(a.shape[1] * 0.5), replace=True)
+        ix2 = np.random.choice(a.shape[2], size=int(a.shape[2] * 0.5), replace=True)
         _test_set_orthogonal_selection_3d(v, a, z, ix0, ix1, ix2)
 
         # sorted increasing
@@ -770,6 +842,27 @@ def test_set_orthogonal_selection_3d():
         _test_set_orthogonal_selection_3d(v, a, z, ix0, ix1, ix2)
 
 
+def test_orthogonal_indexing_fallback_on_get_setitem():
+    z = zarr.zeros((20, 20))
+    z[[1, 2, 3], [1, 2, 3]] = 1
+    np.testing.assert_array_equal(
+        z[:4, :4],
+        [
+            [0, 0, 0, 0],
+            [0, 1, 0, 0],
+            [0, 0, 1, 0],
+            [0, 0, 0, 1],
+        ],
+    )
+    np.testing.assert_array_equal(z[[1, 2, 3], [1, 2, 3]], 1)
+    # test broadcasting
+    np.testing.assert_array_equal(z[1, [1, 2, 3]], [1, 0, 0])
+    # test 1D fancy indexing
+    z2 = zarr.zeros(5)
+    z2[[1, 2, 3]] = 1
+    np.testing.assert_array_equal(z2, [0, 1, 1, 1, 0])
+
+
 def _test_get_coordinate_selection(a, z, selection):
     expect = a[selection]
     actual = z.get_coordinate_selection(selection)
@@ -785,8 +878,8 @@ coordinate_selections_1d_bad = [
     Ellipsis,
     # bad stuff
     2.3,
-    'foo',
-    b'xxx',
+    "foo",
+    b"xxx",
     None,
     (0, 0),
     (slice(None), slice(None)),
@@ -876,10 +969,8 @@ def test_get_coordinate_selection_2d():
     _test_get_coordinate_selection(a, z, (ix0, ix1))
 
     # multi-dimensional selection
-    ix0 = np.array([[1, 1, 2],
-                    [2, 2, 5]])
-    ix1 = np.array([[1, 3, 2],
-                    [1, 0, 0]])
+    ix0 = np.array([[1, 1, 2], [2, 2, 5]])
+    ix1 = np.array([[1, 3, 2], [1, 0, 0]])
     _test_get_coordinate_selection(a, z, (ix0, ix1))
 
     with pytest.raises(IndexError):
@@ -912,7 +1003,6 @@ def _test_set_coordinate_selection(v, a, z, selection):
 
 
 def test_set_coordinate_selection_1d():
-
     # setup
     v = np.arange(1050, dtype=int)
     a = np.empty(v.shape, dtype=v.dtype)
@@ -963,13 +1053,186 @@ def test_set_coordinate_selection_2d():
             _test_set_coordinate_selection(v, a, z, selection)
 
     # multi-dimensional selection
-    ix0 = np.array([[1, 2, 3],
-                    [4, 5, 6]])
-    ix1 = np.array([[1, 3, 2],
-                    [2, 0, 5]])
+    ix0 = np.array([[1, 2, 3], [4, 5, 6]])
+    ix1 = np.array([[1, 3, 2], [2, 0, 5]])
     _test_set_coordinate_selection(v, a, z, (ix0, ix1))
 
 
+def _test_get_block_selection(a, z, selection, expected_idx):
+    expect = a[expected_idx]
+    actual = z.get_block_selection(selection)
+    assert_array_equal(expect, actual)
+    actual = z.blocks[selection]
+    assert_array_equal(expect, actual)
+
+
+block_selections_1d = [
+    # test single item
+    0,
+    5,
+    # test wraparound
+    -1,
+    -4,
+    # test slice
+    slice(5),
+    slice(None, 3),
+    slice(5, 6),
+    slice(-3, -1),
+    slice(None),  # Full slice
+]
+
+block_selections_1d_array_projection = [
+    # test single item
+    slice(100),
+    slice(500, 600),
+    # test wraparound
+    slice(1000, None),
+    slice(700, 800),
+    # test slice
+    slice(500),
+    slice(None, 300),
+    slice(500, 600),
+    slice(800, 1000),
+    slice(None),
+]
+
+block_selections_1d_bad = [
+    # slice not supported
+    slice(3, 8, 2),
+    # bad stuff
+    2.3,
+    "foo",
+    b"xxx",
+    None,
+    (0, 0),
+    (slice(None), slice(None)),
+    [0, 5, 3],
+]
+
+
+def test_get_block_selection_1d():
+    # setup
+    a = np.arange(1050, dtype=int)
+    z = zarr.create(shape=a.shape, chunks=100, dtype=a.dtype)
+    z[:] = a
+
+    for selection, expected_idx in zip(block_selections_1d, block_selections_1d_array_projection):
+        _test_get_block_selection(a, z, selection, expected_idx)
+
+    bad_selections = block_selections_1d_bad + [
+        z.nchunks + 1,  # out of bounds
+        -(z.nchunks + 1),  # out of bounds
+    ]
+
+    for selection in bad_selections:
+        with pytest.raises(IndexError):
+            z.get_block_selection(selection)
+        with pytest.raises(IndexError):
+            z.blocks[selection]
+
+
+block_selections_2d = [
+    # test single item
+    (0, 0),
+    (1, 2),
+    # test wraparound
+    (-1, -1),
+    (-3, -2),
+    # test slice
+    (slice(1), slice(2)),
+    (slice(None, 2), slice(-2, -1)),
+    (slice(2, 3), slice(-2, None)),
+    (slice(-3, -1), slice(-3, -2)),
+    (slice(None), slice(None)),  # Full slice
+]
+
+block_selections_2d_array_projection = [
+    # test single item
+    (slice(300), slice(3)),
+    (slice(300, 600), slice(6, 9)),
+    # test wraparound
+    (slice(900, None), slice(9, None)),
+    (slice(300, 600), slice(6, 9)),
+    # test slice
+    (slice(300), slice(6)),
+    (slice(None, 600), slice(6, 9)),
+    (slice(600, 900), slice(6, None)),
+    (slice(300, 900), slice(3, 6)),
+    (slice(None), slice(None)),  # Full slice
+]
+
+
+def test_get_block_selection_2d():
+    # setup
+    a = np.arange(10000, dtype=int).reshape(1000, 10)
+    z = zarr.create(shape=a.shape, chunks=(300, 3), dtype=a.dtype)
+    z[:] = a
+
+    for selection, expected_idx in zip(block_selections_2d, block_selections_2d_array_projection):
+        _test_get_block_selection(a, z, selection, expected_idx)
+
+    with pytest.raises(IndexError):
+        selection = slice(5, 15), [1, 2, 3]
+        z.get_block_selection(selection)
+    with pytest.raises(IndexError):
+        selection = Ellipsis, [1, 2, 3]
+        z.get_block_selection(selection)
+    with pytest.raises(IndexError):  # out of bounds
+        selection = slice(15, 20), slice(None)
+        z.get_block_selection(selection)
+
+
+def _test_set_block_selection(v: np.ndarray, a: np.ndarray, z: zarr.Array, selection, expected_idx):
+    for value in 42, v[expected_idx], v[expected_idx].tolist():
+        # setup expectation
+        a[:] = 0
+        a[expected_idx] = value
+        # test long-form API
+        z[:] = 0
+        z.set_block_selection(selection, value)
+        assert_array_equal(a, z[:])
+        # test short-form API
+        z[:] = 0
+        z.blocks[selection] = value
+        assert_array_equal(a, z[:])
+
+
+def test_set_block_selection_1d():
+    # setup
+    v = np.arange(1050, dtype=int)
+    a = np.empty(v.shape, dtype=v.dtype)
+    z = zarr.create(shape=a.shape, chunks=100, dtype=a.dtype)
+
+    for selection, expected_idx in zip(block_selections_1d, block_selections_1d_array_projection):
+        _test_set_block_selection(v, a, z, selection, expected_idx)
+
+    for selection in block_selections_1d_bad:
+        with pytest.raises(IndexError):
+            z.set_block_selection(selection, 42)
+        with pytest.raises(IndexError):
+            z.blocks[selection] = 42
+
+
+def test_set_block_selection_2d():
+    # setup
+    v = np.arange(10000, dtype=int).reshape(1000, 10)
+    a = np.empty(v.shape, dtype=v.dtype)
+    z = zarr.create(shape=a.shape, chunks=(300, 3), dtype=a.dtype)
+
+    for selection, expected_idx in zip(block_selections_2d, block_selections_2d_array_projection):
+        _test_set_block_selection(v, a, z, selection, expected_idx)
+
+    with pytest.raises(IndexError):
+        selection = slice(5, 15), [1, 2, 3]
+        z.set_block_selection(selection, 42)
+    with pytest.raises(IndexError):
+        selection = Ellipsis, [1, 2, 3]
+        z.set_block_selection(selection, 42)
+    with pytest.raises(IndexError):  # out of bounds
+        selection = slice(15, 20), slice(None)
+        z.set_block_selection(selection, 42)
+
+
 def _test_get_mask_selection(a, z, selection):
     expect = a[selection]
     actual = z.get_mask_selection(selection)
@@ -985,8 +1248,8 @@ mask_selections_1d_bad = [
     Ellipsis,
     # bad stuff
     2.3,
-    'foo',
-    b'xxx',
+    "foo",
+    b"xxx",
     None,
     (0, 0),
     (slice(None), slice(None)),
@@ -1116,7 +1379,7 @@ def test_get_selection_out():
     # test with different degrees of sparseness
     for p in 0.5, 0.1, 0.01:
         ix0 = np.random.binomial(1, p, size=a.shape[0]).astype(bool)
-        ix1 = np.random.binomial(1, .5, size=a.shape[1]).astype(bool)
+        ix1 = np.random.binomial(1, 0.5, size=a.shape[1]).astype(bool)
         selections = [
             # index both axes with array
             (ix0, ix1),
@@ -1164,22 +1427,20 @@ def test_get_selection_out():
 
 def test_get_selections_with_fields():
 
-    a = [('aaa', 1, 4.2),
-         ('bbb', 2, 8.4),
-         ('ccc', 3, 12.6)]
-    a = np.array(a, dtype=[('foo', 'S3'), ('bar', 'i4'), ('baz', 'f8')])
+    a = [("aaa", 1, 4.2), ("bbb", 2, 8.4), ("ccc", 3, 12.6)]
+    a = np.array(a, dtype=[("foo", "S3"), ("bar", "i4"), ("baz", "f8")])
     z = zarr.create(shape=a.shape, chunks=2, dtype=a.dtype, fill_value=None)
     z[:] = a
 
     fields_fixture = [
-        'foo',
-        ['foo'],
-        ['foo', 'bar'],
-        ['foo', 'baz'],
-        ['bar', 'baz'],
-        ['foo', 'bar', 'baz'],
-        ['bar', 'foo'],
-        ['baz', 'bar', 'foo'],
+        "foo",
+        ["foo"],
+        ["foo", "bar"],
+        ["foo", "baz"],
+        ["bar", "baz"],
+        ["foo", "bar", "baz"],
+        ["bar", "foo"],
+        ["baz", "bar", "foo"],
     ]
 
     for fields in fields_fixture:
@@ -1267,30 +1528,28 @@ def test_get_selections_with_fields():
 
     # missing/bad fields
     with pytest.raises(IndexError):
-        z.get_basic_selection(Ellipsis, fields=['notafield'])
+        z.get_basic_selection(Ellipsis, fields=["notafield"])
     with pytest.raises(IndexError):
         z.get_basic_selection(Ellipsis, fields=slice(None))
 
 
 def test_set_selections_with_fields():
 
-    v = [('aaa', 1, 4.2),
-         ('bbb', 2, 8.4),
-         ('ccc', 3, 12.6)]
-    v = np.array(v, dtype=[('foo', 'S3'), ('bar', 'i4'), ('baz', 'f8')])
+    v = [("aaa", 1, 4.2), ("bbb", 2, 8.4), ("ccc", 3, 12.6)]
+    v = np.array(v, dtype=[("foo", "S3"), ("bar", "i4"), ("baz", "f8")])
     a = np.empty_like(v)
     z = zarr.empty_like(v, chunks=2)
 
     fields_fixture = [
-        'foo',
+        "foo",
         [],
-        ['foo'],
-        ['foo', 'bar'],
-        ['foo', 'baz'],
-        ['bar', 'baz'],
-        ['foo', 'bar', 'baz'],
-        ['bar', 'foo'],
-        ['baz', 'bar', 'foo'],
+        ["foo"],
+        ["foo", "bar"],
+        ["foo", "baz"],
+        ["bar", "baz"],
+        ["foo", "bar", "baz"],
+        ["bar", "foo"],
+        ["baz", "bar", "foo"],
     ]
 
     for fields in fields_fixture:
@@ -1320,8 +1579,8 @@ def test_set_selections_with_fields():
                 key = fields
 
             # setup expectation
-            a[:] = ('', 0, 0)
-            z[:] = ('', 0, 0)
+            a[:] = ("", 0, 0)
+            z[:] = ("", 0, 0)
             assert_array_equal(a, z[:])
             a[key] = v[key]
             # total selection
@@ -1329,31 +1588,31 @@ def test_set_selections_with_fields():
             assert_array_equal(a, z[:])
 
             # basic selection with slice
-            a[:] = ('', 0, 0)
-            z[:] = ('', 0, 0)
+            a[:] = ("", 0, 0)
+            z[:] = ("", 0, 0)
             a[key][0:2] = v[key][0:2]
             z.set_basic_selection(slice(0, 2), v[key][0:2], fields=fields)
             assert_array_equal(a, z[:])
 
             # orthogonal selection
-            a[:] = ('', 0, 0)
-            z[:] = ('', 0, 0)
+            a[:] = ("", 0, 0)
+            z[:] = ("", 0, 0)
             ix = [0, 2]
             a[key][ix] = v[key][ix]
             z.set_orthogonal_selection(ix, v[key][ix], fields=fields)
             assert_array_equal(a, z[:])
 
             # coordinate selection
-            a[:] = ('', 0, 0)
-            z[:] = ('', 0, 0)
+            a[:] = ("", 0, 0)
+            z[:] = ("", 0, 0)
             ix = [0, 2]
             a[key][ix] = v[key][ix]
             z.set_coordinate_selection(ix, v[key][ix], fields=fields)
             assert_array_equal(a, z[:])
 
             # mask selection
-            a[:] = ('', 0, 0)
-            z[:] = ('', 0, 0)
+            a[:] = ("", 0, 0)
+            z[:] = ("", 0, 0)
             ix = [True, False, True]
             a[key][ix] = v[key][ix]
             z.set_mask_selection(ix, v[key][ix], fields=fields)
@@ -1461,17 +1720,24 @@ def test_numpy_int_indexing():
         # 1D test cases
         ((1070,), (50,), [("__getitem__", (slice(200, 400),))]),
         ((1070,), (50,), [("__getitem__", (slice(200, 400, 100),))]),
-        ((1070,), (50,), [
-            ("__getitem__", (slice(200, 400),)),
-            ("__setitem__", (slice(200, 400, 100),)),
-        ]),
-
+        (
+            (1070,),
+            (50,),
+            [
+                ("__getitem__", (slice(200, 400),)),
+                ("__setitem__", (slice(200, 400, 100),)),
+            ],
+        ),
         # 2D test cases
-        ((40, 50), (5, 8), [
-            ("__getitem__", (slice(6, 37, 13), (slice(4, 10)))),
-            ("__setitem__", (slice(None), (slice(None)))),
-        ]),
-    ]
+        (
+            (40, 50),
+            (5, 8),
+            [
+                ("__getitem__", (slice(6, 37, 13), (slice(4, 10)))),
+                ("__setitem__", (slice(None), (slice(None)))),
+            ],
+        ),
+    ],
 )
 def test_accessed_chunks(shape, chunks, ops):
     # Test that only the required chunks are accessed during basic selection operations
@@ -1519,9 +1785,8 @@ def test_accessed_chunks(shape, chunks, ops):
             # don't determine if the chunk was actually partial here, just that the
             # counts are consistent that this might have happened
             if optype == "__setitem__":
-                assert (
-                    ("__getitem__", ci) not in delta_counts or
-                    delta_counts.pop(("__getitem__", ci)) == 1
-                )
+                assert ("__getitem__", ci) not in delta_counts or delta_counts.pop(
+                    ("__getitem__", ci)
+                ) == 1
         # Check that no other chunks were accessed
         assert len(delta_counts) == 0
