@@ -5,16 +5,35 @@ import numbers
 from textwrap import TextWrapper
 import mmap
 import time
+from typing import (
+    Any,
+    Callable,
+    Dict,
+    Iterator,
+    Mapping,
+    Optional,
+    Tuple,
+    TypeVar,
+    Union,
+    Iterable,
+    cast,
+)
 
 import numpy as np
 from asciitree import BoxStyle, LeftAligned
 from asciitree.traversal import Traversal
-from collections.abc import Iterable
-from numcodecs.compat import ensure_text, ensure_ndarray_like
+from numcodecs.compat import (
+    ensure_text,
+    ensure_ndarray_like,
+    ensure_bytes,
+    ensure_contiguous_ndarray_like,
+)
+from numcodecs.ndarray_like import NDArrayLike
 from numcodecs.registry import codec_registry
 from numcodecs.blosc import cbuffer_sizes, cbuffer_metainfo
 
-from typing import Any, Callable, Dict, Optional, Tuple, Union
+KeyType = TypeVar("KeyType")
+ValueType = TypeVar("ValueType")
 
 
 def flatten(arg: Iterable) -> Iterable:
@@ -27,14 +46,13 @@ def flatten(arg: Iterable) -> Iterable:
 
 # codecs to use for object dtype convenience API
 object_codecs = {
-    str.__name__: 'vlen-utf8',
-    bytes.__name__: 'vlen-bytes',
-    'array': 'vlen-array',
+    str.__name__: "vlen-utf8",
+    bytes.__name__: "vlen-bytes",
+    "array": "vlen-array",
 }
 
 
 class NumberEncoder(json.JSONEncoder):
-
     def default(self, o):
         # See json.JSONEncoder.default docstring for explanation
         # This is necessary to encode numpy dtype
@@ -47,35 +65,37 @@ class NumberEncoder(json.JSONEncoder):
 
 def json_dumps(o: Any) -> bytes:
     """Write JSON in a consistent, human-readable way."""
-    return json.dumps(o, indent=4, sort_keys=True, ensure_ascii=True,
-                      separators=(',', ': '), cls=NumberEncoder).encode('ascii')
+    return json.dumps(
+        o, indent=4, sort_keys=True, ensure_ascii=True, separators=(",", ": "), cls=NumberEncoder
+    ).encode("ascii")
 
 
-def json_loads(s: str) -> Dict[str, Any]:
+def json_loads(s: Union[bytes, str]) -> Dict[str, Any]:
     """Read JSON in a consistent way."""
-    return json.loads(ensure_text(s, 'ascii'))
+    return json.loads(ensure_text(s, "utf-8"))
 
 
-def normalize_shape(shape) -> Tuple[int]:
+def normalize_shape(shape: Union[int, Tuple[int, ...], None]) -> Tuple[int, ...]:
     """Convenience function to normalize the `shape` argument."""
 
     if shape is None:
-        raise TypeError('shape is None')
+        raise TypeError("shape is None")
 
     # handle 1D convenience form
     if isinstance(shape, numbers.Integral):
         shape = (int(shape),)
 
     # normalize
+    shape = cast(Tuple[int, ...], shape)
     shape = tuple(int(s) for s in shape)
     return shape
 
 
 # code to guess chunk shape, adapted from h5py
 
-CHUNK_BASE = 256*1024  # Multiplier by which chunks are adjusted
-CHUNK_MIN = 128*1024  # Soft lower limit (128k)
-CHUNK_MAX = 64*1024*1024  # Hard upper limit
+CHUNK_BASE = 256 * 1024  # Multiplier by which chunks are adjusted
+CHUNK_MIN = 128 * 1024  # Soft lower limit (128k)
+CHUNK_MAX = 64 * 1024 * 1024  # Hard upper limit
 
 
 def guess_chunks(shape: Tuple[int, ...], typesize: int) -> Tuple[int, ...]:
@@ -89,12 +109,12 @@ def guess_chunks(shape: Tuple[int, ...], typesize: int) -> Tuple[int, ...]:
 
     ndims = len(shape)
     # require chunks to have non-zero length for all dimensions
-    chunks = np.maximum(np.array(shape, dtype='=f8'), 1)
+    chunks = np.maximum(np.array(shape, dtype="=f8"), 1)
 
     # Determine the optimal chunk size in bytes using a PyTables expression.
     # This is kept as a float.
-    dset_size = np.product(chunks)*typesize
-    target_size = CHUNK_BASE * (2**np.log10(dset_size/(1024.*1024)))
+    dset_size = np.prod(chunks) * typesize
+    target_size = CHUNK_BASE * (2 ** np.log10(dset_size / (1024.0 * 1024)))
 
     if target_size > CHUNK_MAX:
         target_size = CHUNK_MAX
@@ -108,14 +128,14 @@ def guess_chunks(shape: Tuple[int, ...], typesize: int) -> Tuple[int, ...]:
         # 1b. We're within 50% of the target chunk size, AND
         # 2. The chunk is smaller than the maximum chunk size
 
-        chunk_bytes = np.product(chunks)*typesize
+        chunk_bytes = np.prod(chunks) * typesize
 
-        if (chunk_bytes < target_size or
-                abs(chunk_bytes-target_size)/target_size < 0.5) and \
-                chunk_bytes < CHUNK_MAX:
+        if (
+            chunk_bytes < target_size or abs(chunk_bytes - target_size) / target_size < 0.5
+        ) and chunk_bytes < CHUNK_MAX:
             break
 
-        if np.product(chunks) == 1:
+        if np.prod(chunks) == 1:
             break  # Element size larger than CHUNK_MAX
 
         chunks[idx % ndims] = math.ceil(chunks[idx % ndims] / 2.0)
@@ -124,9 +144,7 @@ def guess_chunks(shape: Tuple[int, ...], typesize: int) -> Tuple[int, ...]:
     return tuple(int(x) for x in chunks)
 
 
-def normalize_chunks(
-    chunks: Any, shape: Tuple[int, ...], typesize: int
-) -> Tuple[int, ...]:
+def normalize_chunks(chunks: Any, shape: Tuple[int, ...], typesize: int) -> Tuple[int, ...]:
     """Convenience function to normalize the `chunks` argument for an array
     with the given `shape`."""
 
@@ -146,17 +164,16 @@ def normalize_chunks(
 
     # handle bad dimensionality
     if len(chunks) > len(shape):
-        raise ValueError('too many dimensions in chunks')
+        raise ValueError("too many dimensions in chunks")
 
     # handle underspecified chunks
     if len(chunks) < len(shape):
         # assume chunks across remaining dimensions
-        chunks += shape[len(chunks):]
+        chunks += shape[len(chunks) :]
 
     # handle None or -1 in chunks
     if -1 in chunks or None in chunks:
-        chunks = tuple(s if c == -1 or c is None else int(c)
-                       for s, c in zip(shape, chunks))
+        chunks = tuple(s if c == -1 or c is None else int(c) for s, c in zip(shape, chunks))
 
     return tuple(chunks)
 
@@ -168,30 +185,34 @@ def normalize_dtype(dtype: Union[str, np.dtype], object_codec) -> Tuple[np.dtype
         dtype = dtype.__name__  # type: ignore
     if isinstance(dtype, str):
         # allow ':' to delimit class from codec arguments
-        tokens = dtype.split(':')
+        tokens = dtype.split(":")
         key = tokens[0]
         if key in object_codecs:
             dtype = np.dtype(object)
             if object_codec is None:
                 codec_id = object_codecs[key]
                 if len(tokens) > 1:
-                    args = tokens[1].split(',')
+                    args = tokens[1].split(",")
                 else:
                     args = []
                 try:
                     object_codec = codec_registry[codec_id](*args)
                 except KeyError:  # pragma: no cover
-                    raise ValueError('codec %r for object type %r is not '
-                                     'available; please provide an '
-                                     'object_codec manually' % (codec_id, key))
+                    raise ValueError(
+                        "codec %r for object type %r is not "
+                        "available; please provide an "
+                        "object_codec manually" % (codec_id, key)
+                    )
             return dtype, object_codec
 
     dtype = np.dtype(dtype)
 
     # don't allow generic datetime64 or timedelta64, require units to be specified
-    if dtype == np.dtype('M8') or dtype == np.dtype('m8'):
-        raise ValueError('datetime64 and timedelta64 dtypes with generic units '
-                         'are not supported, please specify units (e.g., "M8[ns]")')
+    if dtype == np.dtype("M8") or dtype == np.dtype("m8"):
+        raise ValueError(
+            "datetime64 and timedelta64 dtypes with generic units "
+            'are not supported, please specify units (e.g., "M8[ns]")'
+        )
 
     return dtype, object_codec
 
@@ -209,16 +230,17 @@ def is_total_slice(item, shape: Tuple[int]) -> bool:
     if item == slice(None):
         return True
     if isinstance(item, slice):
-        item = item,
+        item = (item,)
     if isinstance(item, tuple):
         return all(
-            (isinstance(s, slice) and
-                ((s == slice(None)) or
-                 ((s.stop - s.start == l) and (s.step in [1, None]))))
-            for s, l in zip(item, shape)
+            (
+                isinstance(it, slice)
+                and ((it == slice(None)) or ((it.stop - it.start == sh) and (it.step in [1, None])))
+            )
+            for it, sh in zip(item, shape)
         )
     else:
-        raise TypeError('expected slice or tuple of slices, found %r' % item)
+        raise TypeError("expected slice or tuple of slices, found %r" % item)
 
 
 def normalize_resize_args(old_shape, *args):
@@ -233,33 +255,32 @@ def normalize_resize_args(old_shape, *args):
     else:
         new_shape = tuple(new_shape)
     if len(new_shape) != len(old_shape):
-        raise ValueError('new shape must have same number of dimensions')
+        raise ValueError("new shape must have same number of dimensions")
 
     # handle None in new_shape
-    new_shape = tuple(s if n is None else int(n)
-                      for s, n in zip(old_shape, new_shape))
+    new_shape = tuple(s if n is None else int(n) for s, n in zip(old_shape, new_shape))
 
     return new_shape
 
 
 def human_readable_size(size) -> str:
     if size < 2**10:
-        return '%s' % size
+        return "%s" % size
     elif size < 2**20:
-        return '%.1fK' % (size / float(2**10))
+        return "%.1fK" % (size / float(2**10))
     elif size < 2**30:
-        return '%.1fM' % (size / float(2**20))
+        return "%.1fM" % (size / float(2**20))
     elif size < 2**40:
-        return '%.1fG' % (size / float(2**30))
+        return "%.1fG" % (size / float(2**30))
     elif size < 2**50:
-        return '%.1fT' % (size / float(2**40))
+        return "%.1fT" % (size / float(2**40))
     else:
-        return '%.1fP' % (size / float(2**50))
+        return "%.1fP" % (size / float(2**50))
 
 
 def normalize_order(order: str) -> str:
     order = str(order).upper()
-    if order not in ['C', 'F']:
+    if order not in ["C", "F"]:
         raise ValueError("order must be either 'C' or 'F', found: %r" % order)
     return order
 
@@ -268,8 +289,7 @@ def normalize_dimension_separator(sep: Optional[str]) -> Optional[str]:
     if sep in (".", "/", None):
         return sep
     else:
-        raise ValueError(
-            "dimension_separator must be either '.' or '/', found: %r" % sep)
+        raise ValueError("dimension_separator must be either '.' or '/', found: %r" % sep)
 
 
 def normalize_fill_value(fill_value, dtype: np.dtype):
@@ -277,22 +297,24 @@ def normalize_fill_value(fill_value, dtype: np.dtype):
     if fill_value is None or dtype.hasobject:
         # no fill value
         pass
-    elif fill_value == 0:
+    elif not isinstance(fill_value, np.void) and fill_value == 0:
         # this should be compatible across numpy versions for any array type, including
         # structured arrays
         fill_value = np.zeros((), dtype=dtype)[()]
 
-    elif dtype.kind == 'U':
+    elif dtype.kind == "U":
         # special case unicode because of encoding issues on Windows if passed through numpy
         # https://github.com/alimanfoo/zarr/pull/172#issuecomment-343782713
 
         if not isinstance(fill_value, str):
-            raise ValueError('fill_value {!r} is not valid for dtype {}; must be a '
-                             'unicode string'.format(fill_value, dtype))
+            raise ValueError(
+                "fill_value {!r} is not valid for dtype {}; must be a "
+                "unicode string".format(fill_value, dtype)
+            )
 
     else:
         try:
-            if isinstance(fill_value, bytes) and dtype.kind == 'V':
+            if isinstance(fill_value, bytes) and dtype.kind == "V":
                 # special case for numpy 1.14 compatibility
                 fill_value = np.array(fill_value, dtype=dtype.str).view(dtype)[()]
             else:
@@ -300,8 +322,10 @@ def normalize_fill_value(fill_value, dtype: np.dtype):
 
         except Exception as e:
             # re-raise with our own error message to be helpful
-            raise ValueError('fill_value {!r} is not valid for dtype {}; nested '
-                             'exception: {}'.format(fill_value, dtype, e))
+            raise ValueError(
+                "fill_value {!r} is not valid for dtype {}; nested "
+                "exception: {}".format(fill_value, dtype, e)
+            )
 
     return fill_value
 
@@ -310,7 +334,7 @@ def normalize_storage_path(path: Union[str, bytes, None]) -> str:
 
     # handle bytes
     if isinstance(path, bytes):
-        path = str(path, 'ascii')
+        path = str(path, "ascii")
 
     # ensure str
     if path is not None and not isinstance(path, str):
@@ -319,21 +343,21 @@ def normalize_storage_path(path: Union[str, bytes, None]) -> str:
     if path:
 
         # convert backslash to forward slash
-        path = path.replace('\\', '/')
+        path = path.replace("\\", "/")
 
         # ensure no leading slash
-        while len(path) > 0 and path[0] == '/':
+        while len(path) > 0 and path[0] == "/":
             path = path[1:]
 
         # ensure no trailing slash
-        while len(path) > 0 and path[-1] == '/':
+        while len(path) > 0 and path[-1] == "/":
             path = path[:-1]
 
         # collapse any repeated slashes
         previous_char = None
-        collapsed = ''
+        collapsed = ""
         for char in path:
-            if char == '/' and previous_char == '/':
+            if char == "/" and previous_char == "/":
                 pass
             else:
                 collapsed += char
@@ -341,12 +365,12 @@ def normalize_storage_path(path: Union[str, bytes, None]) -> str:
         path = collapsed
 
         # don't allow path segments with just '.' or '..'
-        segments = path.split('/')
-        if any(s in {'.', '..'} for s in segments):
+        segments = path.split("/")
+        if any(s in {".", ".."} for s in segments):
             raise ValueError("path containing '.' or '..' segment not allowed")
 
     else:
-        path = ''
+        path = ""
 
     return path
 
@@ -358,32 +382,34 @@ def buffer_size(v) -> int:
 def info_text_report(items: Dict[Any, Any]) -> str:
     keys = [k for k, v in items]
     max_key_len = max(len(k) for k in keys)
-    report = ''
+    report = ""
     for k, v in items:
-        wrapper = TextWrapper(width=80,
-                              initial_indent=k.ljust(max_key_len) + ' : ',
-                              subsequent_indent=' '*max_key_len + ' : ')
+        wrapper = TextWrapper(
+            width=80,
+            initial_indent=k.ljust(max_key_len) + " : ",
+            subsequent_indent=" " * max_key_len + " : ",
+        )
         text = wrapper.fill(str(v))
-        report += text + '\n'
+        report += text + "\n"
     return report
 
 
 def info_html_report(items) -> str:
     report = '<table class="zarr-info">'
-    report += '<tbody>'
+    report += "<tbody>"
     for k, v in items:
-        report += '<tr>' \
-                  '<th style="text-align: left">%s</th>' \
-                  '<td style="text-align: left">%s</td>' \
-                  '</tr>' \
-                  % (k, v)
-    report += '</tbody>'
-    report += '</table>'
+        report += (
+            "<tr>"
+            '<th style="text-align: left">%s</th>'
+            '<td style="text-align: left">%s</td>'
+            "</tr>" % (k, v)
+        )
+    report += "</tbody>"
+    report += "</table>"
     return report
 
 
 class InfoReporter:
-
     def __init__(self, obj):
         self.obj = obj
 
@@ -397,24 +423,22 @@ class InfoReporter:
 
 
 class TreeNode:
-
     def __init__(self, obj, depth=0, level=None):
         self.obj = obj
         self.depth = depth
         self.level = level
 
     def get_children(self):
-        if hasattr(self.obj, 'values'):
+        if hasattr(self.obj, "values"):
             if self.level is None or self.depth < self.level:
                 depth = self.depth + 1
-                return [TreeNode(o, depth=depth, level=self.level)
-                        for o in self.obj.values()]
+                return [TreeNode(o, depth=depth, level=self.level) for o in self.obj.values()]
         return []
 
     def get_text(self):
         name = self.obj.name.split("/")[-1] or "/"
-        if hasattr(self.obj, 'shape'):
-            name += ' {} {}'.format(self.obj.shape, self.obj.dtype)
+        if hasattr(self.obj, "shape"):
+            name += " {} {}".format(self.obj.shape, self.obj.dtype)
         return name
 
     def get_type(self):
@@ -422,7 +446,6 @@ class TreeNode:
 
 
 class TreeTraversal(Traversal):
-
     def get_children(self, node):
         return node.get_children()
 
@@ -433,8 +456,8 @@ class TreeTraversal(Traversal):
         return node.get_text()
 
 
-tree_group_icon = 'folder'
-tree_array_icon = 'table'
+tree_group_icon = "folder"
+tree_array_icon = "table"
 
 
 def tree_get_icon(stype: str) -> str:
@@ -481,37 +504,28 @@ def tree_widget(group, expand, level):
 
 
 class TreeViewer:
-
     def __init__(self, group, expand=False, level=None):
 
         self.group = group
         self.expand = expand
         self.level = level
 
-        self.text_kwargs = dict(
-            horiz_len=2,
-            label_space=1,
-            indent=1
-        )
+        self.text_kwargs = dict(horiz_len=2, label_space=1, indent=1)
 
         self.bytes_kwargs = dict(
-            UP_AND_RIGHT="+",
-            HORIZONTAL="-",
-            VERTICAL="|",
-            VERTICAL_AND_RIGHT="+"
+            UP_AND_RIGHT="+", HORIZONTAL="-", VERTICAL="|", VERTICAL_AND_RIGHT="+"
         )
 
         self.unicode_kwargs = dict(
             UP_AND_RIGHT="\u2514",
             HORIZONTAL="\u2500",
             VERTICAL="\u2502",
-            VERTICAL_AND_RIGHT="\u251C"
+            VERTICAL_AND_RIGHT="\u251C",
         )
 
     def __bytes__(self):
         drawer = LeftAligned(
-            traverse=TreeTraversal(),
-            draw=BoxStyle(gfx=self.bytes_kwargs, **self.text_kwargs)
+            traverse=TreeTraversal(), draw=BoxStyle(gfx=self.bytes_kwargs, **self.text_kwargs)
         )
         root = TreeNode(self.group, level=self.level)
         result = drawer(root)
@@ -524,8 +538,7 @@ class TreeViewer:
 
     def __unicode__(self):
         drawer = LeftAligned(
-            traverse=TreeTraversal(),
-            draw=BoxStyle(gfx=self.unicode_kwargs, **self.text_kwargs)
+            traverse=TreeTraversal(), draw=BoxStyle(gfx=self.unicode_kwargs, **self.text_kwargs)
         )
         root = TreeNode(self.group, level=self.level)
         return drawer(root)
@@ -539,16 +552,21 @@ class TreeViewer:
 
 
 def check_array_shape(param, array, shape):
-    if not hasattr(array, 'shape'):
-        raise TypeError('parameter {!r}: expected an array-like object, got {!r}'
-                        .format(param, type(array)))
+    if not hasattr(array, "shape"):
+        raise TypeError(
+            "parameter {!r}: expected an array-like object, got {!r}".format(param, type(array))
+        )
     if array.shape != shape:
-        raise ValueError('parameter {!r}: expected array with shape {!r}, got {!r}'
-                         .format(param, shape, array.shape))
+        raise ValueError(
+            "parameter {!r}: expected array with shape {!r}, got {!r}".format(
+                param, shape, array.shape
+            )
+        )
 
 
 def is_valid_python_name(name):
     from keyword import iskeyword
+
     return name.isidentifier() and not iskeyword(name)
 
 
@@ -581,9 +599,9 @@ class PartialReadBuffer:
         self.read_blocks = set()
 
         _key_path = self.map._key_to_str(store_key)
-        _key_path = _key_path.split('/')
+        _key_path = _key_path.split("/")
         _chunk_path = [self.chunk_store._normalize_key(_key_path[-1])]
-        _key_path = '/'.join(_key_path[:-1] + _chunk_path)
+        _key_path = "/".join(_key_path[:-1] + _chunk_path)
         self.key_path = _key_path
 
     def prepare_chunk(self):
@@ -595,21 +613,15 @@ class PartialReadBuffer:
         self.buff[0:16] = header
         self.nblocks = nbytes / blocksize
         self.nblocks = (
-            int(self.nblocks)
-            if self.nblocks == int(self.nblocks)
-            else int(self.nblocks + 1)
+            int(self.nblocks) if self.nblocks == int(self.nblocks) else int(self.nblocks + 1)
         )
         if self.nblocks == 1:
             self.buff = self.read_full()
             return
-        start_points_buffer = self.fs.read_block(
-            self.key_path, 16, int(self.nblocks * 4)
-        )
-        self.start_points = np.frombuffer(
-            start_points_buffer, count=self.nblocks, dtype=np.int32
-        )
+        start_points_buffer = self.fs.read_block(self.key_path, 16, int(self.nblocks * 4))
+        self.start_points = np.frombuffer(start_points_buffer, count=self.nblocks, dtype=np.int32)
         self.start_points_max = self.start_points.max()
-        self.buff[16: (16 + (self.nblocks * 4))] = start_points_buffer
+        self.buff[16 : (16 + (self.nblocks * 4))] = start_points_buffer
         self.n_per_block = blocksize / typesize
 
     def read_part(self, start, nitems):
@@ -639,12 +651,33 @@ class PartialReadBuffer:
         return self.chunk_store[self.store_key]
 
 
-def retry_call(callabl: Callable,
-               args=None,
-               kwargs=None,
-               exceptions: Tuple[Any, ...] = (),
-               retries: int = 10,
-               wait: float = 0.1) -> Any:
+class UncompressedPartialReadBufferV3:
+    def __init__(self, store_key, chunk_store, itemsize):
+        assert chunk_store.supports_efficient_get_partial_values
+        self.chunk_store = chunk_store
+        self.store_key = store_key
+        self.itemsize = itemsize
+
+    def prepare_chunk(self):
+        pass
+
+    def read_part(self, start, nitems):
+        return self.chunk_store.get_partial_values(
+            [(self.store_key, (start * self.itemsize, nitems * self.itemsize))]
+        )[0]
+
+    def read_full(self):
+        return self.chunk_store[self.store_key]
+
+
+def retry_call(
+    callabl: Callable,
+    args=None,
+    kwargs=None,
+    exceptions: Tuple[Any, ...] = (),
+    retries: int = 10,
+    wait: float = 0.1,
+) -> Any:
     """
     Make several attempts to invoke the callable. If one of the given exceptions
     is raised, wait the given period of time and retry up to the given number of
@@ -656,7 +689,7 @@ def retry_call(callabl: Callable,
     if kwargs is None:
         kwargs = {}
 
-    for attempt in range(1, retries+1):
+    for attempt in range(1, retries + 1):
         try:
             return callabl(*args, **kwargs)
         except exceptions:
@@ -696,3 +729,63 @@ def all_equal(value: Any, array: Any):
             # using == raises warnings from numpy deprecated pattern, but
             # using np.equal() raises type errors for structured dtypes...
             return np.all(value == array)
+
+
+def ensure_contiguous_ndarray_or_bytes(buf) -> Union[NDArrayLike, bytes]:
+    """Convenience function to coerce `buf` to ndarray-like array or bytes.
+
+    First check if `buf` can be zero-copy converted to a contiguous array.
+    If not, `buf` will be copied to a newly allocated `bytes` object.
+
+    Parameters
+    ----------
+    buf : ndarray-like, array-like, or bytes-like
+        A numpy array like object such as numpy.ndarray, cupy.ndarray, or
+        any object exporting a buffer interface.
+
+    Returns
+    -------
+    arr : NDArrayLike or bytes
+        A ndarray-like or bytes object
+    """
+
+    try:
+        return ensure_contiguous_ndarray_like(buf)
+    except TypeError:
+        # An error is raised if `buf` couldn't be zero-copy converted
+        return ensure_bytes(buf)
+
+
+class ConstantMap(Mapping[KeyType, ValueType]):
+    """A read-only map that maps all keys to the same constant value
+
+    Useful if you want to call `getitems()` with the same context for all keys.
+
+    Parameters
+    ----------
+    keys
+        The keys of the map. Will be copied to a frozenset if it isn't already.
+    constant
+        The constant that all keys are mapping to.
+    """
+
+    def __init__(self, keys: Iterable[KeyType], constant: ValueType) -> None:
+        self._keys = keys if isinstance(keys, frozenset) else frozenset(keys)
+        self._constant = constant
+
+    def __getitem__(self, key: KeyType) -> ValueType:
+        if key not in self._keys:
+            raise KeyError(repr(key))
+        return self._constant
+
+    def __iter__(self) -> Iterator[KeyType]:
+        return iter(self._keys)
+
+    def __len__(self) -> int:
+        return len(self._keys)
+
+    def __contains__(self, key: object) -> bool:
+        return key in self._keys
+
+    def __repr__(self) -> str:
+        return repr({k: v for k, v in self.items()})
